@@ -3,7 +3,7 @@ db_table target_schema[27];
 
 /* ADDRESS table */
 struct address a;
-db_field address_rec[9] = {
+db_field address_rec[10] = {
    {ADDRESS_RVA,"rva", TYPE_ULONG, 1, ADDRESS_RVA, 0, 0, 
       "", &address_rec[1]},
    {PA,"pa", TYPE_ULONG, 1, PA, 0, OFFSET(a,pa), 
@@ -21,7 +21,8 @@ db_field address_rec[9] = {
    {ADDRESS_STRUCTURE,"structure", TYPE_ULONG, 1, 0,STRUCTURE,OFFSET(a,structure), 
       "", &address_rec[8]},
    {ADDRESS_COMMENT,"comment", TYPE_ULONG, 1, 0, COMMENT, OFFSET(a,comment), 
-      "", NULL}
+      "", &address_rec[9]},
+   {ADDRESS_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(a,tag), "", NULL}
 };
 
 db_index address_idx[] = {
@@ -31,7 +32,7 @@ db_index address_idx[] = {
 
 /* SECTION table */
 struct section s;
-db_field section_rec[4] = {
+db_field section_rec[5] = {
    {SECTION_NAME,"name", TYPE_CHAR, 32, SECTION_NAME, 0, 0, 
       "", &section_rec[1]},
    {SECTION_RVA,"rva", TYPE_ULONG, 1, SECTION_RVA, ADDRESS, OFFSET(s,rva), 
@@ -39,6 +40,8 @@ db_field section_rec[4] = {
    {SECTION_SIZE,"size", TYPE_ULONG, 1, 0, 0, OFFSET(s,size), 
       "", &section_rec[3]},
    {SECTION_FLAGS,"flags", TYPE_ULONG, 1, 0, 0, OFFSET(s,flags), 
+	"",  &section_rec[4]},
+   {SECTION_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(s,tag), 
       "", NULL}
 };
 
@@ -70,7 +73,7 @@ db_index xref_idx[] = {
 
 /* CODE table */
 struct code c;
-db_field code_rec[10] = {
+db_field code_rec[11] = {
    {CODE_RVA,"rva", TYPE_ULONG, 1, CODE_RVA, ADDRESS, 0, 
       "", &code_rec[1]},
    {CODE_FUNC, "func", TYPE_ULONG, 1, CODE_FUNCTION, FUNCTION, 0, 
@@ -90,7 +93,8 @@ db_field code_rec[10] = {
    {SRCTYPE,"srcType", TYPE_ULONG, 1, 0, 0, OFFSET(c,srcType), 
       "", &code_rec[9]},
    {AUXTYPE,"auxType", TYPE_ULONG, 1, 0, 0, OFFSET(c,auxType), 
-      "", NULL},
+      "", &code_rec[10]},
+   {CODE_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(c,tag), "", NULL},
 
 };
 
@@ -190,7 +194,7 @@ db_index sysref_idx[3] = {
 
 /* INT_CODE table */
 struct int_code ic;
-db_field intcode_rec[12] = {
+db_field intcode_rec[13] = {
    {INT_CODE_ID,"id", TYPE_ULONG, 1, INT_CODE_ID, 0, 0, 
       "", &intcode_rec[1]},
    {OPCODE,"opcode", TYPE_ULONG, 1, 0, 0, OFFSET(ic, opcode), 
@@ -214,7 +218,8 @@ db_field intcode_rec[12] = {
    {INT_CODE_RVA,"rva", TYPE_ULONG, 1, 0, 0, OFFSET(ic, rva), 
       "", &intcode_rec[11]},
    {INT_CODE_COMMENT,"comment", TYPE_ULONG, 1, 0, 0, OFFSET(ic, comment), 
-      "", NULL}
+      "", &intcode_rec[12]},
+   {INT_CODE_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(ic, tag), "", NULL}
 };
 
 db_index intcode_idx[] = {
@@ -322,13 +327,14 @@ db_index name_idx[] = {
 
 /* COMMENT table */
 struct comment cmt;
-db_field comment_rec[3] = {
+db_field comment_rec[4] = {
    {COMMENT_ID,"id", TYPE_ULONG, 1, COMMENT_ID, 0, 0, 
       "", &comment_rec[1]},
    {COMMENT_TYPE,"type", TYPE_ULONG, 1, 0, 0, OFFSET(cmt,type), 
       "", &comment_rec[2]},
    {COMMENT_TEXT,"text", TYPE_CHAR, 256, 0, 0, OFFSET(cmt,text), 
-      "", NULL}
+      "", &comment_rec[3]},
+   {COMMENT_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(cmt,tag), "", NULL}
 };
 
 db_index comment_idx[] = {
@@ -337,7 +343,7 @@ db_index comment_idx[] = {
 
 /* FUNCTION table */
 struct function f;
-db_field func_rec[5] = {
+db_field func_rec[6] = {
    {FUNCTION_ID,"id", TYPE_ULONG, 1, FUNCTION_ID, 0, 0, 
       "", &func_rec[1]},
    {FUNCTION_RVA,"rva", TYPE_ULONG, 1, FUNCTION_RVA, ADDRESS, OFFSET(f,rva), 
@@ -347,7 +353,8 @@ db_field func_rec[5] = {
    {RET_TYPE,"ret_type", TYPE_ULONG, 1, 0, DATA_TYPE, OFFSET(f,ret_type), 
       "", &func_rec[4]},
    {FUNCTION_COMMENT,"comment", TYPE_ULONG, 1, 0, COMMENT, OFFSET(f,comment), 
-      "", NULL}
+      "", &func_rec[5]},
+   {FUNCTION_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(f,tag), "", NULL}
 };
 
 db_index func_idx[] = {
@@ -427,7 +434,7 @@ db_index feffect_idx[] = {
 
 /* F_INLINE table */
 struct f_inline fi;
-db_field finline_rec[4] = {
+db_field finline_rec[5] = {
    {F_INLINE_ID,"id", TYPE_ULONG, 1, F_INLINE_ID, 0, 0, 
       "", &finline_rec[1]},
    {F_INLINE_NAME,"name", TYPE_CHAR, 64, 0, 0, OFFSET(fi,name), 
@@ -435,7 +442,8 @@ db_field finline_rec[4] = {
    {F_INLINE_RVA,"rva", TYPE_ULONG, 1, F_INLINE_RVA, ADDRESS, OFFSET(fi,rva), 
       "", &finline_rec[3]},
    {F_INLINE_SIZE,"size", TYPE_USHORT, 1, 0, 0, OFFSET(fi,size), 
-      "", NULL}
+      "", &finline_rec[4]},
+   {F_INLINE_TAG,"tag", TYPE_USHORT, 1, 0, 0, OFFSET(fi,tag), "", NULL}
 };
 
 db_index finline_idx[] = {
