@@ -46,6 +46,13 @@ struct DB_DESCRIPTOR {
 
 extern struct DB_DESCRIPTOR *db_head, *db_current, *db_config, *db_target;
 
+struct BDB_ITERATOR {
+	int index;
+	int db_state;
+	BDB_ITERATOR_FN func;
+	void *arg;
+};
+
 int db_init();
 void db_init_seq();
 int db_load( char *dbname, char *filename, struct DB_DESCRIPTOR *db );
