@@ -8,10 +8,11 @@
 
 typedef int (*sprint_fn)(char *, int, int);
 typedef int (*genfinal_fn)(int);
-typedef int (*genhllfile_fn)(char *);
+typedef int (*genhllfile_fn)(FILE *);
 
 struct EXT__LANG {            /* target HighLevelLanguage */
    struct EXTENSION ext;
+   int options;
    /* function pointers */
    void *fn_parsehdr;   /* process header for constants, fn protos, etc */
    sprint_fn fn_sprint_proto;
