@@ -203,7 +203,7 @@ int addr_new(long rva, int size, long pa, int flags)
  * for rva, resizing other ADDRESS objects as needed.
  */
 	struct address addr_old= {0}, address = {0};
-	void *state;
+	void *state = NULL;
 	int done = 0;
 	
 
@@ -227,8 +227,11 @@ int addr_new(long rva, int size, long pa, int flags)
 	if (!flags)
 		flags = ADDR_DATA;
 	
-	if (pa == -1 || pa > (long)target.info.size) 
+	if (pa == ADDR_NO_PA) {
+		pa = ADDR_INVALID;
+	} else if (pa == ADDR_UNK_PA || pa > (long)target.info.size) {
 		pa = (long) addr_guess_pa( rva );
+	}
 
 	/* fill address structure */
 	address.rva = rva;
