@@ -24,10 +24,9 @@ int target_set_header(char *strHeader)
  */
 	if (target.info.header)
 		free(target.info.header);
-	target.info.header = malloc(strlen(strHeader) + 1);
+	target.info.header = strdup(strHeader);
 	if (!target.info.header)
 		return (sys_set_lasterr(9050));
-	strcpy(target.info.header, strHeader);
 	disasm_env.flags |= DB_MOD;
 	return (1);
 }
@@ -37,7 +36,7 @@ char *target_header()
 	return (target.info.header);
 }
 
-int deal_with_addr(unsigned long rva, struct code *code, int xrefType,
+static int deal_with_addr(unsigned long rva, struct code *code, int xrefType,
 		   int nametype)
 {
 	int addrType;
@@ -74,7 +73,7 @@ int deal_with_addr(unsigned long rva, struct code *code, int xrefType,
 	return (1);
 }
 
-int analyze_curr_op(struct code *code, long *op, int *type)
+static int analyze_curr_op(struct code *code, long *op, int *type)
 {
 /* This finds absolute and rfelative address references in an opcode,
  * creates an address object for that reference, creates an xref to
@@ -112,7 +111,7 @@ int analyze_curr_op(struct code *code, long *op, int *type)
 		/* RELATIVE OFFSET Convert offset to RVA */
 		/* NOTE: skip rel offsets of 0 */
 		if (*op && (a = GetAddressObject(code->rva))) {
-			/* add IMM value to end of current instruction [rva + size] */
+			/* add IMM value to end of current insn [rva + size] */
 			rva = *op + code->rva + a->size;
 
 			/* note the change in instruction comment */
@@ -127,9 +126,9 @@ int analyze_curr_op(struct code *code, long *op, int *type)
 
 		} else if (*op) {
 			/* This shouldn't happen */
-			DEBUG_PrintMsg("Horrible error in analyze_curr_op! "
-				       "Addr %x for CODE struct doesn't exist\n",
-				       code->rva);;
+			DEBUG_PrintMsg("Horrible error in analyze_curr_op! Addr"
+				       " %x for CODE struct doesn't exist\n",
+				       code->rva);
 			break;
 		}
 		/* fall through to handle new address */
@@ -142,15 +141,15 @@ int analyze_curr_op(struct code *code, long *op, int *type)
 /* TODO: For DATA, associate default pointer type with pointer rva */
 /* TODO: For DATA, associate default data type with deferenced rva */
 		rva = *op;
-		/* first, create an address object for the pointer itself */
+		/* first, create address object for pointer itself */
 		deal_with_addr(rva, code, xrefType, NAME_NEWPTR);
 
-		/* next, create an address for the rva contained in the pointer */
+		/* next, create address for rva contained in the pointer */
 
-		/* find section that rva is in, get offset from start of section,
-		 * read bytes from file, use *that* as rva.Put op in brackets. */
+		/* find section rva is in, get offset from start of section,
+		 * read bytes from file, use that as rva.Put op in brackets. */
 		if ((rva = addr_pa(rva)) && rva < target.info.size) {
-			/* get address in pointer from target image, if possible */
+			/* get addr in pointer from target image, if possible */
 			encpy((char *) &rva, target.image + rva,
 			      ext_arch->sz_addr);
 			if (addr_is_valid(rva))
@@ -172,7 +171,7 @@ int analyze_curr_op(struct code *code, long *op, int *type)
 	return (1);
 }
 
-int handle_code_vm( struct code *code ) {
+static int handle_code_vm( struct code *code ) {
 	/* here, we want to handle the stack for the VM, and
 	 * also to disassemble_forward for all dynamic calls. 
 	 * OK, this really should only be called from disasm_forward,
@@ -190,10 +189,12 @@ int handle_code_vm( struct code *code ) {
 				vm_read_reg( code->dest, &data );
 				if ( ! data.flags ) {
 					/* is this appropriate? Useful tho */
-					xref_new(code->rva, data.u.l, XREF_EXEC);
+					xref_new(code->rva, data.u.l, 
+							XREF_EXEC);
 					/* disasm forward from val in reg */
-					if ( disasm_follow_xrefs )
+					if ( disasm_follow_xrefs ){
 						disasm_forward( data.u.l );
+					}
 				//} else if ( data.flags & VM_STACK_PTR ) {
 				}
 			}
@@ -217,22 +218,24 @@ int handle_code_vm( struct code *code ) {
 		case INS_MOV:
 			/* all we really care about is MOVs into registers */
 			if ( (code->destType & OP_TYPE_MASK) == OP_REG ) {
-				if ( (code->srcType & OP_TYPE_MASK) == OP_REG ) {
+				if ((code->srcType & OP_TYPE_MASK) == OP_REG) {
 					vm_read_reg( code->dest, &data );
 					vm_write_reg( code->dest, &data );
-				} else if ( (code->srcType & OP_TYPE_MASK) == OP_IMM ||
-						(code->srcType & OP_TYPE_MASK) == OP_ADDR ) {
+				} else if ( (code->srcType & OP_TYPE_MASK) == 
+						OP_IMM ||
+					    (code->srcType & OP_TYPE_MASK) == 
+						OP_ADDR ) {
 					data.u.l = code->src;
 					vm_write_reg( code->dest, &data );
 				}
 			}
 		default:
-			;
+			break;
 	}
 	return(1);
 }
 
-int analyze_curr_inst(struct code *code)
+static int analyze_curr_inst(struct code *code)
 {
 	struct code_effect e;
 	/* add support for arch-specific comments, e.g. */
@@ -240,8 +243,8 @@ int analyze_curr_inst(struct code *code)
 
 	/* Get standard reg effects */
 	code_effect_gen( code );
-	/* Get Operand Types */
 
+	/* Get Operand Types */
 	if (code->destType)
 		analyze_curr_op(code, &code->dest, &(code->destType));
 	if (code->srcType)
@@ -277,18 +280,22 @@ int disasm_target(char *disasm, void *param)
 	/* called with libdisasm.$DISASM.so as 'disasm',
 	 * e.g. "libdisasm.full.so would be "full" */
 	char fullname[256] = { 0 };
+	void *plugin;
 
 
 	DEBUG_PrintMsg("Disassemble %s on TARGET ", disasm);
 	DEBUG_PrintMsg("entry %08X\n", target.info.entry);
 	sprintf(fullname, "disasm.%s", disasm);
-	if (plugin_load(fullname))
-		ext_plugin_main(param);
+	plugin = plugin_load(fullname, 0);
+	if (plugin) {
+		ext_plugin_main(plugin, param);
+		plugin_unload(plugin);
+	}
 /* else error */
 	return (1);
 }
 
-int disasm_address(long pa, struct code *c, long rva)
+int disasm_address(unsigned long pa, struct code *c, unsigned long rva)
 {
 /* pa  = offset from image start  
  * c   = code struct filled with info
@@ -306,14 +313,12 @@ int disasm_address(long pa, struct code *c, long rva)
 				code_del( code.rva );
 			} else {
 				return( sys_set_lasterr(4120) );
-				//return( a.size );
 			}
 		} else if ( a.rva <= rva && (a.rva + a.size) > rva ) {
 			if ( disasm_prefs.options & DISASM_REDO ) {
 				code_del( code.rva );
 			} else {
 				return( sys_set_lasterr(4130) );
-				//return(a.size - (rva - a.rva));
 			}
 		}
 	}
@@ -366,7 +371,7 @@ int disasm_section(char *name)
 	return (1);
 }
 
-int disasm_range(long rva, int size)
+int disasm_range(unsigned long rva, int size)
 {
 /* rva must have an address record */
 	struct address a;
@@ -397,47 +402,56 @@ int disasm_range(long rva, int size)
 	return (1);
 }
 
+static unsigned long disasm_branch_dest( struct code *c, int size ) {
+	int type;
+	unsigned long rva = 0;
+
+	if ( ! c ) 	return(0);
+	type = c->destType & OP_TYPE_MASK;
+	switch ( type ) {
+		case OP_REG:
+			/* skip registers for now */
+			break;
+		case OP_PTR:
+			rva = c->dest;
+			/* 1. get pa for rva */
+			if ((rva = addr_pa(rva)) && rva < target.info.size) {
+				/* 2. get dword [pointer] stored at pa */
+				encpy((char *) &rva, target.image + rva, 
+						ext_arch->sz_addr);
+			}
+			break;
+		case OP_IMM:
+		case OP_REL:
+			rva = c->rva + size + c->dest;
+			break;
+		default:
+			rva = c->dest;
+	}
+	return( rva );
+}
+
 int disasm_addr_switch(struct code *c, int size)
 {
-	int cont = 1, destType;
+	int cont = 1;
 	char tmp_op[32];	/* temporary operand */
-	long new_rva;
+	long new_rva = 0;
 
 	/* At this point, addr_new and AnalyzeInst have been done */
-	new_rva = 0;
-	destType = c->destType & 0x0F00;
 	switch (c->mnemType & INS_TYPE_MASK) {
 		case INS_BRANCH:
 			cont = 0;
 		case INS_BRANCHCC:
 		case INS_CALL:
 		case INS_CALLCC:
-			//printf("Branch Type %d ", code.Type);
-			if (destType == OP_REG) {
-				/* skip registers for now */
-				;
-			} else if (destType == OP_PTR) {
-				new_rva = c->dest;
-				/* 1. get pa for rva */
-				if ((new_rva = addr_pa(new_rva)) &&
-				    new_rva < target.info.size) {
-					/* 2. get dword [pointer] stored at pa */
-					encpy((char *) &new_rva, target.image
-					      + new_rva, ext_arch->sz_addr);
-				}
-			} else if (destType == OP_IMM || destType == OP_REL) {
-				/* convert relative offset to address - */
-				new_rva = c->rva + size + c->dest;
-			} else {
-				new_rva = c->dest;
-			}
+			new_rva = disasm_branch_dest( c, size );
 			break;
 		case INS_RET:
 		case INS_TRET:
 			cont = 0;
 		default:
 			break;
-	}			// end switch
+	}
 
 	/* Follow branch if one was detected */
 	sys_visual("\b-");		/* step 3 on this stack level */
@@ -462,9 +476,10 @@ int disasm_forward_addr(struct address *a, struct section *s)
 		sys_visual("\b|");	/* step 1 of this stack level */
 		size = disasm_address(a->pa + bytes, &code, code.rva);
 		sys_visual("\b/");	/* step 2 on this stack level */
-		if (size < 1)
+		if (size < 1) {
 			cont = 0;
-		else {
+			continue;
+		} else {
 			bytes += size;
 			cont = disasm_addr_switch(&code, size);
 		}
@@ -477,12 +492,12 @@ int disasm_forward_addr(struct address *a, struct section *s)
 	return (1);
 }
 
-int disasm_forward(long rva)
+int disasm_forward(unsigned long rva)
 {
-	struct section sec;
+	struct section sec = {0}, snext = {0};
 	struct address addr;
 	char buf[32];
-	int *state;
+	void *state;
 
 	disasm_follow_xrefs = 1;
 	if ((ext_arch->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
@@ -500,16 +515,21 @@ int disasm_forward(long rva)
 		/* this happens with GNU linkers. Basically, we find the start
 		 * of the next section and create an address linking the end of
 		 * this section with the start of the next one */
-		bdb_find_closest_next(SECTION_RVA, &rva, &sec);
+		bdb_find_closest_prev(SECTION_RVA, &rva, &sec);
+		bdb_find_closest_next(SECTION_RVA, &rva, &snext); 
+		if ( snext.rva <= sec.rva ) {
+			/* this is not a valid address in the program */
+			db_restore_state(state);
+			return(sys_set_lasterr(4250));
+		}
 		bdb_find_closest_prev(ADDRESS_RVA, &rva, &addr);
 		/* create new address object on the fly */
 		addr.rva += addr.size; 
 		addr.pa += addr.size;
-		addr.size = sec.rva - addr.rva;
+		addr.size = snext.rva - addr.rva;
 		memset( &addr.dataType, 0, 5 * sizeof(long));
 		bdb_record_insert( ADDRESS, &addr );
 		/* resize section */
-		bdb_find_closest_prev(SECTION_RVA, &rva, &sec);
 		sec.size += addr.size;
 		bdb_record_update(SECTION_RVA, &sec.rva, &sec);
 		disasm_forward_addr(&addr, &sec);
@@ -521,7 +541,7 @@ int disasm_forward(long rva)
 
 
 /* Convenience functions -- low-level wrappers to the disasm_lib fn's */
-unsigned long disasm_find_pat_in_range(long rva, int size, int type)
+unsigned long disasm_find_pat_in_range(unsigned long rva, int size, int type)
 {
 	struct address a;
 	int cont;
@@ -535,7 +555,7 @@ unsigned long disasm_find_pat_in_range(long rva, int size, int type)
 	return (0);
 }
 
-unsigned long disasm_find_func_in_range(long rva, int size)
+unsigned long disasm_find_func_in_range(unsigned long rva, int size)
 {
 	unsigned long start;
 	struct address a;
@@ -547,12 +567,14 @@ unsigned long disasm_find_func_in_range(long rva, int size)
 		if ( a.rva + a.size >= start ) {
 			/* there is an address directly before this one */
 			if ( bdb_index_find( CODE_RVA, &a.rva, &c) ) {
-				if ( (c.mnemType & INS_TYPE_MASK) == INS_RET  ||
+				if ( (c.mnemType & INS_TYPE_MASK) == INS_RET    ||
 				 	 c.mnemType & INS_TYPE_MASK == INS_BRANCH ||
-				  	 c.mnemType & INS_TYPE_MASK == INS_TRET ) {
+				  	 c.mnemType & INS_TYPE_MASK == INS_TRET   ||
+					 c.mnemType & INS_TYPE_MASK == INS_NOP     ) {
 					/* previous CODE is a JMP, RET or IRET */
 					return( start );
 				}
+				/* else we are in the middle of code or data! */
 			} else 		/* no preceding code */
 				return(start);
 		} else			/* no preceding address */
@@ -566,23 +588,54 @@ unsigned long disasm_find_func_in_range(long rva, int size)
 	}
 }
 
-unsigned long disasm_find_func_end(long rva)
-{
-	struct function f;
+static unsigned long disasm_find_epilogue(unsigned long rva, unsigned long id){
 	struct address a;
-	int cont;
-
-
-	cont = bdb_find_closest_prev(ADDRESS_RVA, &rva, &a);
-	bdb_index_find(FUNCTION_RVA, &a.rva, &f);
-
+	struct code c;
+	int cont, type;
+	unsigned long branch_end = 0, next_end = 0;
+	
+	cont = bdb_index_find(ADDRESS_RVA, &rva, &a);
 	while (cont) {
-		code_set_func(a.rva, f.id);
+		/* while we're here, aissgn fn to the code */
+		code_set_func(a.rva, id);
 		if (ext_code_pat(a.rva, FUNCTION_EPILOGUE))
 			return (a.rva);
-		cont = bdb_index_next(ADDRESS_RVA, &a);
+		/* do we need to branch or anything? */
+		bdb_index_find( CODE_RVA, &a.rva, &c );
+		type = c.mnemType & INS_TYPE_MASK;
+		if ( type == INS_BRANCH ) {
+			rva = disasm_branch_dest( &c , a.size);
+			if ( rva ) {
+				cont = bdb_index_find( ADDRESS_RVA, &rva, &a );
+			} else {
+				/* cannot follow jump */
+				return( rva );
+			}
+		} else if ( type == INS_BRANCHCC ) {
+			if ( bdb_index_next(ADDRESS_RVA, &a) ) {
+				next_end = disasm_find_epilogue( a.rva, id );
+			}
+			rva = disasm_branch_dest( &c, a.size );
+			if ( rva && rva > next_end ) {
+				branch_end = disasm_find_epilogue( rva, id );
+			}
+			return( (branch_end > next_end)? branch_end : next_end );
+		} else if ( type == INS_RET ) {
+			return( a.rva );
+		} else {
+			cont = bdb_index_next(ADDRESS_RVA, &a);
+		}
 	}
-	return (0);
+	return(0);
+}
+
+unsigned long disasm_find_func_end(unsigned long rva)
+{
+	struct function f;
+
+	if( bdb_find_closest_prev(FUNCTION_RVA, &rva, &f) ) 
+		return( disasm_find_epilogue( f.rva, f.id ) );
+	return(0);
 }
 
 int disasm_byte_order()
