@@ -20,6 +20,11 @@ int target_fix_path( char *path, char *buf, int len ) {
 		} else {
 			snprintf( buf, len, "%s/%s", getenv("HOME"), &path[2] );
 		}
+	} else if (path[0] == '.' && path[1] != '.') {
+		path++;
+
+		getcwd(buf, len);
+		strncat(buf, path, len - strlen(buf) - 1);
 	} else {
 		strncpy( buf, path, len );
 	}
@@ -32,12 +37,12 @@ int target_path_info(char *path)
 
 	/*    make target.info.path and target.info.name */
 	c = strrchr(path, '/');
-	if (c != NULL) {
-		strncpy(target.info.path, path, c - path + 1);
+	if (c) {
+		strncpy(target.info.path, path, c - path);
 		strncpy(target.info.name, (c + 1), sizeof (target.info.name));
 		target.info.path[c - path + 1] = 0;	/* terminate string */
 	} else {
-		strncpy(target.info.path, "./", 3);
+		getcwd(target.info.path, PATH_MAX);
 		strncpy(target.info.name, path, sizeof (target.info.name));
 	}
 	return (1);
@@ -50,7 +55,7 @@ int target_new()
 	struct stat tmpStat;
 
 	/*   Copy target binary to .bdb */
-	sprintf(buf, "cp -f %s%s %s/.%s.bdb/%s", target.info.path,
+	sprintf(buf, "cp -f %s/%s %s/.%s.bdb/%s", target.info.path,
 		target.info.name, disasm_env.dbpath, target.info.name,
 		target.info.name);
 	if (!((tmp = popen(buf, "r")) && (pclose(tmp) == 0))) {
@@ -62,7 +67,7 @@ int target_new()
 		target.info.name);
 	target.fd = open(buf, O_RDWR);
 	if (target.fd == -1) {
-		sprintf(buf, "Cannot open file %s", errno);
+		sprintf(buf, "Cannot open file %s", strerror(errno));
 		sys_errmsg(9030, buf);
 		return (sys_set_lasterr(9030));
 	}
@@ -84,7 +89,9 @@ int target_new()
 	//addr_new(0, target.info.size, 0, ADDR_DATA);
 
 	disasm_env.flags |= DB_LOADED || DB_MOD;
-	target.status = DISASM_TGT_LOADED;
+	/* clear all flags but extensions [user may have hand-loaded ext's] */
+	target.status &= 0x0FF0;
+	target.status |= DISASM_TGT_LOADED;
 	return (1);
 }
 
@@ -128,11 +135,6 @@ int target_load(char *path) {
 
 	DEBUG_PrintMsg("Loading TARGET filename: %s\n", path);
 
-	/* clear target struct and set entry to -1 */
-	memset((char *) &target, 0, sizeof (struct DISASM_TGT));
-	env_target_defaults();
-	target.info.entry = 0xFFFFFFFF;
-
 	/* Make sure no DB is currently loaded */
 	if (disasm_env.flags)
 		target_close_db();
@@ -188,13 +190,17 @@ int target_load_bdb(char *dbname)
 	// Fill Target Struct
 	sprintf(buf, "%s/.%s/.info", disasm_env.dbpath, target.info.dbname);
 	f = fopen(buf, "r");
+
 	if (f) {
-		fscanf(f, "%s\n%s\n%x\n%lx\n%s\n%s\n%s\n%s\n%s\n%s\n%x\n%x\n",
-		       target.info.name, target.info.path, &target.info.size,
-		       &target.info.entry, target.arch.name,
-		       target.assembler.name, target.comp.name,
-		       target.format.name, target.lang.name, target.os.name,
-		       &target.status, &target.info.endian);
+		fscanf(f, "%s\n%s\n%x\n%lx\n%x\n%x\n%s\n%s\n%f\n%s",
+			target.info.name, target.info.path, &target.info.size,
+			&target.info.entry, &target.info.endian, &target.info.type,
+			target.info.vendor, target.info.model, &target.info.version,
+			target.info.comment);
+		fscanf(f, "%s\n%s\n%s\n%s\n%s\n%s\n%x\n",
+			target.arch.name, target.assembler.name, target.comp.name,
+			target.format.name, target.lang.name, target.os.name,
+			&target.status);
 		fclose(f);
 	} else
 		return (sys_set_lasterr(4561));
@@ -249,12 +255,15 @@ void target_save_db_as(char *name)
 	DEBUG_PrintVar(buf, "%s");
 	f = fopen(buf, "w");
 	if (f) {
-		fprintf(f, "%s\n%s\n%x\n%lx\n%s\n%s\n%s\n%s\n%s\n%s\n%x\n%x\n",
+		fprintf(f, "%s\n%s\n%x\n%lx\n%x\n%x\n%s\n%s\n%f\n%s",
 			target.info.name, target.info.path, target.info.size,
-			target.info.entry, target.arch.name,
-			target.assembler.name, target.comp.name,
+			target.info.entry, target.info.endian, target.info.type,
+			target.info.vendor, target.info.model, target.info.version,
+			target.info.comment);
+		fprintf(f, "%s\n%s\n%s\n%s\n%s\n%s\n%x\n",
+			target.arch.name, target.assembler.name, target.comp.name,
 			target.format.name, target.lang.name, target.os.name,
-			target.status, target.info.endian);
+			target.status);
 		fclose(f);
 	} else {
 		sys_errmsg(4561, "Unable to create .info file!");
@@ -363,6 +372,7 @@ void target_close_db()
 	if (disasm_env.flags & TGT_MAPPED) {
 		close(target.fd);	/* cleanup mmap'ed image */
 		munmap(target.image, target.info.size);
+		target.image = NULL;
 		disasm_env.flags ^= TGT_MAPPED;
 	}
 	if (target.info.header)
@@ -375,10 +385,13 @@ void target_close_db()
 	if (!((tmp = popen(buf, "r")) && (pclose(tmp) == 0))) {
 		sys_errmsg(4511, "Error cleaning up .bdb");
 	}
-	target.image = NULL;
+	/* refill target struct with defaults */
+	memset((char *) &target, 0, sizeof (struct DISASM_TGT));
+	env_target_defaults();
+
 	if (!(disasm_env.flags & QUIT_NOW)) {
 		DEBUG_Print("target_close_db:  sys_re_init\n");
-		sys_re_init(disasm_env.home);
+		sys_re_init(disasm_env.base);
 	}
 	DEBUG_Print("target_close_db: done\n");
 	return;
@@ -386,31 +399,31 @@ void target_close_db()
 
 int target_save_asm(char *filename)
 {
-	return( target_output_as("asm", filename) );
+	return( target_output_as("asm", filename, 0) );
 }
 
 int target_save_hll(char *filename)
 {
-	return( target_output_as("hll", filename) );
+	return( target_output_as("hll", filename, 0) );
 }
 
 int target_save_lst(char *filename)
 {
-	return( target_output_as("lst", filename) );
+	return( target_output_as("lst", filename, 0) );
 }
 
 int target_save_hex(char *filename)
 {
-	return( target_output_as("hex", filename) );
+	return( target_output_as("hex", filename, 0) );
 }
 
 int target_save_diff(char *filename)
 {
-	return( target_output_as("diff", filename) );
+	return( target_output_as("diff", filename, 0) );
 }
 int target_save_binary(char *filename)
 {
-	return( target_output_as("binary", filename) );
+	return( target_output_as("binary", filename, 0) );
 }
 
 int target_set_ext(char *format, char *arch, char *comp, char *os,
@@ -440,49 +453,36 @@ int target_set_format(char *file_format, int options)
  * such as relocations/imports/exports, should be dealt with by the
  * parser as well.
  */
-	char libname[PATH_MAX];
+	char libname[PATH_MAX] = {0};
 	struct stat statbuf;
 
 	DEBUG_PrintMsg("Setting TARGET format:  %s\n", file_format);
-	/* if .bc not exist, assume .so */
-	sprintf(libname, "%s/formats/%s.bc", disasm_env.home, file_format);
-	if (stat(libname, &statbuf))
-		/* assume shared library ... load_extension will catch missing .so */
-		sprintf(libname, "%s/formats/lib%s.so", disasm_env.home,
-			file_format);
-	if (ext_format->ext.filename
-	    && ! strcmp(libname, ext_format->ext.filename)) {
-		//unload_extension( EXT_FORMAT );
-		//return(1);
-		;
-	}
+	if ( ! ext_gen_filename( "formats", file_format, libname, PATH_MAX) )
+		return(sys_set_lasterr(4594));
+
+	if ( ext_format->ext.filename && 
+	     ! strcmp(libname, ext_format->ext.filename)) 
+		return(1);
+
 	DEBUG_PrintVar(libname, "%s");
-	if ((ext_format->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		if (!load_extension(EXT_FORMAT, libname, ext_format))
-			return (sys_set_lasterr(4594));
+	ext_format->options = options;
+	if (!load_extension(EXT_FORMAT, libname, ext_format))
+		return (sys_set_lasterr(4594));
 
 	strncpy(target.format.name, file_format, 32);
-	if ((ext_format->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		target.status = DISASM_TGT_FORMAT_SET;
+	target.status |= DISASM_TGT_FORMAT_SET;
 	return (1);
 }
 
 int target_apply_format()
 {
-	/* if (! strcmp( target.format.name, "MAGIC") ) {
-	 * 	for each line in formats/MAGIC
-	 * 		if ( ! strncmp(line.magic, target.image[0] )
-	 * 			strcpy( target.format.name, line.name)
-	 * 			break;
-	 * 	if ! found default to ELF
-	 */
 	if (! ext_format->ext.filename[0] ) 
 		return( sys_set_lasterr(4594) );
 	/* call the read_header routine */
 	ext_read_header();
 	disasm_env.flags |= DB_MOD;
 	/* move this into a master PhasesOfDisassembly controller */
-	target.status = DISASM_TGT_PREDISASM;
+	target.status |= DISASM_TGT_PREDISASM;
 	return (1);
 
 }
@@ -492,31 +492,20 @@ int target_set_arch(char *file_arch, int options)
 	char libname[PATH_MAX];
 	struct stat tmpStat;
 
-	//DEBUG_PrintMsg("Setting TARGET arch:  %s\n", file_arch);
 	/* default to the current disassembler */
 	if (file_arch == NULL)
 		strcpy(libname, ext_arch->ext.filename);
-	else {
-		/* if the current disassembler != disasm, unload it */
-		sprintf(libname, "%s/arch/%s.bc", disasm_env.home, file_arch);
-		if (stat(libname, &tmpStat))
-			/* if .bc not exist, assume .so */
-			sprintf(libname, "%s/arch/lib%s.so", disasm_env.home,
-				file_arch);
-	}
-
-	if (stat(libname, &tmpStat) != 0)
-		/* the user really hosed things up; force the i386 library */
-		sprintf(libname, "%s/arch/libi386.so", disasm_env.home);
+	else if ( ! ext_gen_filename( "arch", file_arch, libname, PATH_MAX) )
+		return(sys_set_lasterr(4591));
 
 	/* If the disassembler isn't loaded, load it */
-	if ((ext_arch->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		if (!load_extension(EXT_ARCH, libname, ext_arch))
-			return (sys_set_lasterr(4291));
+	ext_arch->options = options;
+	if (!load_extension(EXT_ARCH, libname, ext_arch))
+		return (sys_set_lasterr(4291));
 
 	strncpy(target.arch.name, file_arch, 32);
 	target.info.endian = ext_arch->endian;
-	disasm_env.flags |= DB_MOD;
+	target.status |= DISASM_TGT_ARCH_SET;
 	return (1);
 }
 int target_set_asm(char *asm_output, int options)
@@ -525,17 +514,13 @@ int target_set_asm(char *asm_output, int options)
 	struct stat statbuf;
 
 	DEBUG_PrintMsg("Setting TARGET assembler: %s\n", asm_output);
-	/* if .bc not exist, assume .so */
-	sprintf(libname, "%s/asm/%s.bc", disasm_env.home, asm_output);
-	if (stat(libname, &statbuf))
-		/* assume shared library ... load_extension will catch missing .so */
-		sprintf(libname, "%s/asm/lib%s.so", disasm_env.home,
-			asm_output);
+	if ( ! ext_gen_filename( "asm", asm_output, libname, PATH_MAX) )
+		return(sys_set_lasterr(4592));
 
-	if (ext_asm->ext.filename && ! strcmp(libname, ext_asm->ext.filename)) {
-		;		//Already loaded: do nothing
-	} else if ((ext_asm->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		if (!load_extension(EXT_ASM, libname, ext_asm))
+	if (ext_asm->ext.filename && ! strcmp(libname, ext_asm->ext.filename)) 
+		return(1);
+	ext_asm->options = options;
+	if (!load_extension(EXT_ASM, libname, ext_asm))
 			return (sys_set_lasterr(4592));
 
 	/* this mechanism will be replaced by something better.... */
@@ -544,7 +529,7 @@ int target_set_asm(char *asm_output, int options)
 		env_tty_data(ext_asm->data_ttyColor);
 	}
 	strncpy(target.assembler.name, asm_output, 32);
-	target.status = DISASM_TGT_ASM_SET;
+	target.status |= DISASM_TGT_ASM_SET;
 	return (1);
 }
 
@@ -554,23 +539,19 @@ int target_set_comp(char *compiler, int options)
 	struct stat statbuf;
 
 	DEBUG_PrintMsg("Setting TARGET Comp: %s\n", compiler);
-	/* if .bc not exist, assume .so */
-	sprintf(libname, "%s/comp/%s.bc", disasm_env.home, compiler);
-	if (stat(libname, &statbuf))
-		/* assume shared library */
-		sprintf(libname, "%s/comp/lib%s.so", disasm_env.home, compiler);
+	if ( ! ext_gen_filename( "comp", compiler, libname, PATH_MAX) )
+		return(sys_set_lasterr(4597));
 
 	if (ext_comp->ext.filename && ! strcmp(libname, ext_comp->ext.filename))
-		//unload_extension( EXT_COMP );
 		return (1);
-	if ((ext_comp->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		if (!load_extension(EXT_COMP, libname, ext_comp))
-			return (sys_set_lasterr(4595));
+	ext_comp->options = options;
+	if (!load_extension(EXT_COMP, libname, ext_comp))
+		return (sys_set_lasterr(4597));
 
 	/* TODO: need some method of determining target bitness */
 	strncpy(target.comp.name, compiler, 32);
 	ext_add_data_types(4);
-	target.status = DISASM_TGT_COMP_SET;
+	target.status |= DISASM_TGT_COMP_SET;
 	return (1);
 }
 
@@ -580,21 +561,17 @@ int target_set_os(char *os, int options)
 	struct stat statbuf;
 
 	DEBUG_PrintMsg("Setting TARGET os: %s\n", os);
-	/* if .bc not exist, assume .so */
-	sprintf(libname, "%s/os/%s.bc", disasm_env.home, os);
-	if (stat(libname, &statbuf))
-		/* assume shared library */
-		sprintf(libname, "%s/os/lib%s.so", disasm_env.home, os);
+	if ( ! ext_gen_filename( "os", os, libname, PATH_MAX) )
+		return(sys_set_lasterr(4598));
 
 	if (ext_os->ext.filename && ! strcmp(libname, ext_os->ext.filename))
-		//unload_extension( EXT_OSNG );
 		return (1);
-	if ((ext_os->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		if (!load_extension(EXT_OS, libname, ext_os))
-			return (sys_set_lasterr(4595));
+	ext_os->options = options;
+	if (!load_extension(EXT_OS, libname, ext_os))
+		return (sys_set_lasterr(4598));
 
 	strncpy(target.os.name, os, 32);
-	target.status = DISASM_TGT_OS_SET;
+	target.status |= DISASM_TGT_OS_SET;
 	return (1);
 }
 
@@ -604,67 +581,88 @@ int target_set_lang(char *language, int options)
 	struct stat statbuf;
 
 	DEBUG_PrintMsg("Setting TARGET Lang: %s\n", language);
-	/* if .bc not exist, assume .so */
-	sprintf(libname, "%s/lang/%s.bc", disasm_env.home, language);
-	if (stat(libname, &statbuf))
-		/* assume shared library */
-		sprintf(libname, "%s/lang/lib%s.so", disasm_env.home, language);
+	if ( ! ext_gen_filename( "lang", language, libname, PATH_MAX) )
+		return(sys_set_lasterr(4595));
 
 	if (ext_lang->ext.filename && ! strcmp(libname, ext_lang->ext.filename))
-		//unload_extension( EXT_LANG );
 		return (1);
-	if ((ext_lang->ext.flags & EXT_STATUS_MASK) != EXT_INIT)
-		if (!load_extension(EXT_LANG, libname, ext_lang))
-			return (sys_set_lasterr(4595));
+
+	ext_lang->options = options;
+	if (!load_extension(EXT_LANG, libname, ext_lang))
+		return (sys_set_lasterr(4595));
 
 	strncpy(target.lang.name, language, 32);
-	target.status = DISASM_TGT_LANG_SET;
+	target.status |= DISASM_TGT_LANG_SET;
 	return (1);
 }
 
-int plugin_load(char *name)
+void * plugin_load(char *name, int options)
 {
 	char libname[PATH_MAX];
 	struct stat statbuf;
+	void * plugin;
 
+	if (! name )	return(0);
 	/* if .bc not exist, assume .so */
-	sprintf(libname, "%s/plugins/%s.bc", disasm_env.home, name);
-	if (stat(libname, &statbuf))
-		/* assume shared library */
-		sprintf(libname, "%s/plugins/lib%s.so", disasm_env.home, name);
+	if ( ! ext_gen_filename( "plugins", name, libname, PATH_MAX) )
+		return((void *)sys_set_lasterr(4596));
 
-	if (ext_plugin->ext.filename
-	    && ! strcmp(libname, ext_plugin->ext.filename)) return (1);
-	if (!load_extension(EXT_PLUGIN, libname, ext_plugin))
-		return (sys_set_lasterr(4596));
-	/* we need the base name for doing plugins, later */
-	strncpy(ext_plugin->name, name, 64);
+	plugin = ext_plugin_load( libname, options );
+	
+	if (! plugin)
+		return ( (void *) (sys_set_lasterr(4596)));
 
-	return (1);
+	return (plugin);
 }
 
-int plugin_exec(void *param)
+int plugin_exec_main(void *plugin, void *param)
 {
-	return (ext_plugin_main(param));
+	if (! plugin) 	return(0);
+	return (ext_plugin_main(plugin, param));
 }
 
-int plugin_unload()
+int plugin_exec( void *plugin, char *name, void *param ) {
+	if (! plugin || ! name) 	return(0);
+	return (ext_plugin_exec( plugin, name, param));
+}
+
+int plugin_unload( void * plugin )
 {
-	return (unload_extension(EXT_PLUGIN));
+	if (! plugin) 	return(0);
+	return( ext_plugin_unload(plugin) );
 }
 
-int target_output_as(char *output, char *filename)
+int target_output_as(char *output, char *filename, int options)
 {
 	/* called with liboutput.$OUTPUT.so as 'output',
 	 * e.g. "liboutput.asm.so would be "asm" */
 	char fullname[256];
+	void *plugin;
+	FILE *f;
 
 
 	DEBUG_PrintMsg("OUTPUT TARGET as %s\n", output);
 	sprintf(fullname, "output.%s", output);
-	if (plugin_load(fullname))
-		ext_plugin_main(filename);
-/* else error */
+
+	/* open output file for writing */
+	if ( filename ) {
+		f = fopen(filename, "w");
+		if ( ! f )
+			return (sys_set_lasterr(9030));
+
+	} else {
+		f = stdout;
+	}
+	plugin = plugin_load(fullname, options);
+	if ( plugin ) {
+		plugin_exec_main( plugin, f );
+		plugin_unload( plugin );
+	}
+	/* else error */
+
+	if ( filename ) {
+		fclose( f );
+	}
 	return (1);
 }
 int target_set_size(int size);
