@@ -24,17 +24,17 @@ int main(int argc, char **argv)
 {
 	struct db_schema *schema;
 	struct db_table *table;
-	char home[PATH_MAX];
+	char base[PATH_MAX];
 	int x;
 
 	if (argc < 2)
 		return (print_usage());
 
 	/* init bastard */
-	if (!sys_find_home(home, PATH_MAX, argv))
+	if (!sys_find_base(base, PATH_MAX, argv))
 		return (-1);
 
-	sys_init(home);
+	sys_init(base);
 	env_set_option( "QUIET" );
 
 	/* get bdb filename */
