@@ -11,17 +11,17 @@ int print_usage()
 
 int main(int argc, char **argv)
 {
-	char home[PATH_MAX];
+	char base[PATH_MAX];
 	int x;
 
 	if (argc < 2)
 		return (print_usage());
 
 	/* init bastard */
-	if (!sys_find_home(home, PATH_MAX, argv))
+	if (!sys_find_base(base, PATH_MAX, argv))
 		return (-1);
 
-	sys_init(home);
+	sys_init(base);
 
 	/* get bdb filename */
 	target_load_bdb(argv[1]);
