@@ -3,6 +3,7 @@
  * Contains routines for interfacing with the Typhoon database module. Typhoon
  * is pretty self-sufficient, so there will not be much here. 
  */
+#include <limits.h>
 #include <sys/stat.h>
 #include <db/db.h>
 #include <db/typhoon.h>
@@ -55,11 +56,19 @@ struct DB_TBLDESC bdb_tbl_desc[] = {
 
 int db_init()
 {
-	d_dbdpath("./db");
-	d_dbfpath("./db");
+	char dbdpath[PATH_MAX];
+	struct stat s;
+
+	sprintf( dbdpath, "%s/db", env_get_share() );
+	d_dbdpath(dbdpath);
+	d_dbfpath(dbdpath);
+	strncat( dbdpath, "/bdb.dbd", PATH_MAX );
+	if ( stat( dbdpath, &s ) ) {
+		return( sys_set_lasterr( 1995 ) );
+	}
 	memset(&seq, 0, sizeof (struct DB_SEQUENCE));
 	db_target = &__db_target;
-	return (0);
+	return (1);
 }
 
 void db_init_seq()
@@ -196,10 +205,11 @@ int db_load(char *dbname, char *filename, struct DB_DESCRIPTOR *db)
 	struct stat s;
 	char db_path[256], cmdBuf[256], *pos;
 
-	if (!db)
-		return (0);
+	if (!db) {
+		return (sys_set_lasterr(1997));
+	}
 	/* set location of .dbd files --> $BASTARD_HOME */
-	snprintf(db_path, 256, "%s/db", disasm_env.home);
+	snprintf(db_path, 256, "%s/db", disasm_env.share);
 	d_dbdpath(db_path);
 
 	pos = strrchr(filename, '/');
@@ -224,9 +234,11 @@ int db_load(char *dbname, char *filename, struct DB_DESCRIPTOR *db)
 		DEBUG_PrintVar(cmdBuf, "%s");
 		if (!((tmp = popen(cmdBuf, "r")) && (pclose(tmp) == 0))) {
 			DEBUG_PrintMsg("Unable to execute cmd %s\n", cmdBuf);
-			return (0);
+			return (sys_set_lasterr(1996));
 		}
 	} else {
+/* NOTE this is the cause of rez's bug: if target_load_bdb() is called, and
+ * the BDB does not exist, we end up here and create the dir by mistake */
 		/* 3. Create directory and such */
 		if (mkdir(db_path, 0777) == -1) {
 			DEBUG_PrintMsg("Unable to mkdir %s\n", db_path);
@@ -263,10 +275,7 @@ int db_load(char *dbname, char *filename, struct DB_DESCRIPTOR *db)
 		if (d_dbset(db_old) == S_INVPARM)
 			printf("Invalid ID loading DB %s\n", dbname);
 
-
-	if (!db->db_id)
-		return (1);	/* hack ;) */
-	return (db->db_id);
+	return ( (db->db_id < 1) ? 1 : db->db_id);
 }
 
 int db_switch(int db)
@@ -292,7 +301,7 @@ int db_unload(struct DB_DESCRIPTOR *db)
 	int tmp;
 
 	if (!db)
-		return (0);
+		return (sys_set_lasterr(1997));
 	tmp = db_switch(db->db_id);
 
 	d_close();
@@ -309,20 +318,20 @@ int db_unload(struct DB_DESCRIPTOR *db)
  * it is assumed that anyone who uses these functions knows what they are
  * doing. */
 
-void *db_save_state()
+void *db_save_state(void)
 {
-	DB_ADDR *curr_rec;
-	curr_rec = (DB_ADDR *) calloc(sizeof (DB_ADDR), 1);
-	d_crget(curr_rec);
-	return (curr_rec);
+	/* internally, state switched to an int */
+	int state = d_stateinit();
+	if (state) 	
+		d_statesave(state);
+	return((void *)state);
 }
 
 int db_restore_state(void *state)
 {
-	DB_ADDR *curr_rec = (DB_ADDR *) state;
-	d_crset(curr_rec);
-	free(curr_rec);
-	return (1);
+	d_staterest((int) state);
+	return( d_statefree((int) state) );
+	return 1;
 }
 
 int db_error()
@@ -338,7 +347,55 @@ void db_error_msg()
 	return;
 }
 
-/* TODO : keep track of curr_rec in these */
+void * bdb_iterator_new(int index, BDB_ITERATOR_FN func, void *arg){
+	struct BDB_ITERATOR *i = calloc(sizeof(struct BDB_ITERATOR), 1);
+	if (i) {
+		i->index = index;
+		i->func = func;
+		i->arg = arg;
+		i->db_state = d_stateinit();
+		d_statesave(i->db_state);
+	}
+	return(i);
+}
+
+int bdb_iterator_next(void *it, void *dest) {
+	struct BDB_ITERATOR *i = (struct BDB_ITERATOR *)it;
+	if (! it || ! dest) return(0);
+
+	/* restore state */
+	d_staterest(i->db_state);
+
+	/* get next record in index */
+	d_keynext(i->index);
+	if ( db_status != S_OKAY ||
+		d_recread(dest) != S_OKAY ) {
+		return(0);
+	}
+
+	/* if a comparison function is provided, use it */
+	if ( i->func ) {
+		while ( db_status == S_OKAY && 
+			 ! (*i->func)(dest, i->arg) ){
+			d_keynext(i->index);
+			d_recread(dest);
+		}
+	}
+
+	/* save state for next iteration */
+	d_statesave(i->db_state);
+	return(1);
+}
+
+int bdb_iterator_free(void *it) {
+	struct BDB_ITERATOR *i = (struct BDB_ITERATOR *)it;
+	if (it) {
+		d_statefree(i->db_state);
+		free(it);
+	}
+	return(1);
+}
+
 int bdb_index_first(int index, void *dest)
 {
 	if (d_keyfrst(index) == S_OKAY) {
@@ -484,21 +541,25 @@ int bdb_find_closest_prev(int index, void *value, void *dest)
 
 	state = db_save_state();
 	if (d_keyfrst(index) == S_NOTFOUND) {
-		/* no need to restore state, table is empty */
+		db_restore_state(state);
 		return(0);
 	}
 	if ( d_keyfind(index, value) == S_OKAY) {
 		found = 1;
 	} else if ( db_status == S_NOTFOUND ) {
-		found = -1;
+		/* guarantee we have a next key */
 		if ( d_keynext(index) == S_NOTFOUND) {
 			d_keylast(index);
 		}
-		if ( d_keyprev(index) != S_OKAY ) {
+		/* now get the previous form the next */
+		if ( db_status == S_OKAY && d_keyprev(index) != S_OKAY ) {
 			d_keyfrst(index);
 		}
+		if ( db_status == S_OKAY ) {
+			found = -1;		/* code marking an inexact result */
+		}
 	}
-	if (found) d_recread(dest);
+	if (found != 0) d_recread(dest);
 	db_restore_state(state);
 	return (found);
 }
@@ -510,43 +571,38 @@ int bdb_find_closest_next(int index, void *value, void *dest)
 
 	state = db_save_state();
 	if (d_keyfrst(index) == S_NOTFOUND) {
-		/* no need to restore state, table is empty */
+		db_restore_state(state);
 		return(0);
 	}
 	if ( d_keyfind(index, value) == S_OKAY) {
 		found = 1;
 	} else if ( db_status == S_NOTFOUND ) {
-		found = -1;
 		if ( d_keynext(index) == S_NOTFOUND) {
 			d_keylast(index);
 		}
+		if ( db_status == S_OKAY ) {
+			found = -1;
+		}
 	}
-	if ( found) d_recread(dest);
+	if (found != 0) d_recread(dest);
 	db_restore_state(state);
 	return (found);
 }
 
 int bdb_table_count(int table)
 {
-	int size, cont, count = 0;
-	char *record;
+	unsigned long count = 0;
+
+	count = d_records(table, &count);
 
-	/* this is a cheat: bc_macro is the largest DB struct by far */
-	size = sizeof (struct bc_macro);
-	record = calloc(size, 1);
-	cont = bdb_table_first(table, record);
-	while (cont) {
-		count++;
-		cont = bdb_table_next(table, record);
-	}
-	free(record);
 	return (count);
 }
 
 
 void bdb_index_foreach(int index, BAPI_CALLBACK func, void *data)
 {
-	int size, cont, *state;
+	int size, cont;
+	void *state;
 	char *record;
 
 	size = sizeof (struct bc_macro);	/* again, a cheat */
