@@ -1,28 +1,49 @@
 #define IAMNOTEIC
 #include <eic.h>
+#include <bastard.h>
+#include <vm.h>
 #ifndef PATH_MAX
 	#define PATH_MAX 4096
 #endif
-#include <bastard.h>
-#include <vm.h>
 
 
 /* wrapper functions */
 static val_t eic_db_save_state(void)
 {
 	val_t v;
-	v.p.sp = v.p.p = db_save_state();
-	v.p.ep = (char *) v.p.p + (sizeof (int));
-	/* ret: void * args:  */
+	v.up = db_save_state();
 	return (v);
 }
 static val_t eic_db_restore_state(void)
 {
 	val_t v;
 	v.ival = db_restore_state(arg(0, getargs(), void *));
-	/* ret: int args: void *state */
 	return (v);
 }
+static val_t eic_bdb_iterator_new(void)
+{
+	val_t v;
+	v.up = bdb_iterator_new( arg(0, getargs(), int), 
+			arg(1, getargs(), void *),
+			arg(2, getargs(), void *));
+	return (v);
+}
+static val_t eic_bdb_iterator_next(void)
+{
+	val_t v;
+	v.ival =
+	    bdb_iterator_next(arg(0, getargs(), void *), 
+			    arg(1, getargs(), void *));
+	return (v);
+}
+static val_t eic_bdb_iterator_free(void)
+{
+	val_t v;
+	v.ival =
+	    bdb_iterator_free(arg(0, getargs(), void *)); 
+	return (v);
+}
+
 static val_t eic_bdb_index_first(void)
 {
 	val_t v;
@@ -420,27 +441,78 @@ static val_t eic_target_output_as(void)
 {
 	val_t v;
 	v.ival = target_output_as(arg(0, getargs(), char *),
-					arg(1, getargs(), char *));
+				  arg(1, getargs(), char *),
+				  arg(2, getargs(), int));
 	/* ret: int args:  char *plugin  */
 	return (v);
 }
+static val_t eic_target_set_vendor(void) {
+	val_t v;
+	target_set_vendor(arg(0, getargs(), char *) );
+	return (v);
+}
+static val_t eic_target_set_model(void) {
+	val_t v;
+	target_set_model(arg(0, getargs(), char *) );
+	return (v);
+}
+static val_t eic_target_set_comment(void) {
+	val_t v;
+	target_set_comment(arg(0, getargs(), char *) );
+	return (v);
+}
+static val_t eic_target_set_version(void) {
+	val_t v;
+	target_set_version(arg(0, getargs(), float) );
+	return (v);
+}
+static val_t eic_target_vendor(void) {
+	val_t v;
+	v.up = target_vendor();
+	return (v);
+}
+static val_t eic_target_model(void) {
+	val_t v;
+	v.up = target_model();
+	return (v);
+}
+static val_t eic_target_comment(void) {
+	val_t v;
+	v.up = target_comment();
+	return (v);
+}
+static val_t eic_target_version(void) {
+	val_t v;
+	v.fval = target_version();
+	return (v);
+}
 static val_t eic_plugin_load(void)
 {
 	val_t v;
-	v.ival = plugin_load(arg(0, getargs(), char *));
+	v.up = plugin_load(arg(0, getargs(), char *),
+	                   arg(1, getargs(), int) );
 	/* ret: int args:  char *plugin  */
 	return (v);
 }
 static val_t eic_plugin_exec(void)
 {
 	val_t v;
-	v.ival = plugin_exec(arg(0, getargs(), void *));
+	v.ival = plugin_exec(arg(0, getargs(), char *),
+				   arg(1, getargs(), char *),
+				   arg(2, getargs(), void *) );
+	return (v);
+}
+static val_t eic_plugin_exec_main(void)
+{
+	val_t v;
+	v.ival = plugin_exec_main(arg(0, getargs(), char *),
+					  arg(1, getargs(), void *)	);
 	return (v);
 }
 static val_t eic_plugin_unload(void)
 {
 	val_t v;
-	v.ival = plugin_unload();
+	v.ival = plugin_unload( arg(0, getargs(), char *));
 	return (v);
 }
 static val_t eic_target_save_db(void)
@@ -1686,18 +1758,29 @@ static val_t eic_env_get_env(void)
 	v.up = env_get_env();
 	return (v);
 }
-static val_t eic_env_set_home(void)
+static val_t eic_env_set_base(void)
 {
 	val_t v;
-	v.ival = env_set_home(arg(0, getargs(), char *));
+	v.ival = env_set_base(arg(0, getargs(), char *));
 	/* ret: char * args:  */
 	return (v);
 }
-static val_t eic_env_get_home(void)
+static val_t eic_env_get_base(void)
 {
 	val_t v;
-	v.up = env_get_home();
-	/* ret: char * args:  */
+	v.up = env_get_base();
+	return (v);
+}
+static val_t eic_env_get_inc(void)
+{
+	val_t v;
+	v.up = env_get_inc();
+	return (v);
+}
+static val_t eic_env_get_share(void)
+{
+	val_t v;
+	v.up = env_get_share();
 	return (v);
 }
 static val_t eic_env_get_prefs(void)
@@ -1899,6 +1982,13 @@ static val_t eic_sec_foreach_addr(void)
 				  arg(2, getargs(), void *));
 	return (v);
 }
+static val_t eic_func_foreach(void)
+{
+	val_t v;
+	v.ival = func_foreach( arg(0, getargs(), void *),
+				   arg(1, getargs(), void *));
+	return (v);
+}
 static val_t eic_func_foreach_addr(void)
 {
 	val_t v;
@@ -2115,6 +2205,9 @@ struct EIC_WRAPPER_TBL {
 	{
 	"db_save_state", eic_db_save_state}, {
 	"db_restore_state", eic_db_restore_state}, {
+	"bdb_iterator_new", eic_bdb_iterator_new}, {
+	"bdb_iterator_next", eic_bdb_iterator_next}, {
+	"bdb_iterator_free", eic_bdb_iterator_free}, {
 	/* BDB manipulation routines */
 	"bdb_index_first", eic_bdb_index_first}, {
 	"bdb_index_next", eic_bdb_index_next}, {
@@ -2167,9 +2260,18 @@ struct EIC_WRAPPER_TBL {
 	"target_set_os", eic_target_set_os}, {
 	"target_set_comp", eic_target_set_comp}, {
 	"target_set_ext", eic_target_set_ext}, {
+	"target_set_vendor", eic_target_set_vendor}, {
+	"target_set_model", eic_target_set_model}, {
+	"target_set_comment", eic_target_set_comment}, {
+	"target_set_version", eic_target_set_version}, {
+	"target_vendor", eic_target_vendor}, {
+	"target_model", eic_target_model}, {
+	"target_comment", eic_target_comment}, {
+	"target_version", eic_target_version}, {
 	"target_output_as", eic_target_output_as}, {
 	"plugin_load", eic_plugin_load}, {
 	"plugin_exec", eic_plugin_exec}, {
+	"plugin_exec_main", eic_plugin_exec_main}, {
 	"plugin_unload", eic_plugin_unload}, {
 	"target_save_db", eic_target_save_db}, {
 	"target_save_bak", eic_target_save_bak}, {
@@ -2336,8 +2438,10 @@ struct EIC_WRAPPER_TBL {
 	"intcode_sprintf", eic_intcode_sprintf}, {
 	"env_get_debugger", eic_env_get_debugger}, {
 	"env_get_env", eic_env_get_env}, {
-	"env_set_home", eic_env_set_home}, {
-	"env_get_home", eic_env_get_home}, {
+	"env_set_base", eic_env_set_base}, {
+	"env_get_base", eic_env_get_base}, {
+	"env_get_inc", eic_env_get_inc}, {
+	"env_get_share", eic_env_get_share}, {
 	"env_get_prefs", eic_env_get_prefs}, {
 	"env_get_target", eic_env_get_target}, {
 	"env_get_editor", eic_env_get_editor}, {
@@ -2355,6 +2459,17 @@ struct EIC_WRAPPER_TBL {
 	"env_defaults", eic_env_defaults}, {
 	"env_pref_defaults", eic_env_pref_defaults}, {
 	"env_target_defaults", eic_env_target_defaults}, {
+	"addr_foreach_xref", eic_addr_foreach_xref}, {
+	"sec_foreach", eic_sec_foreach}, {
+	"sec_foreach_addr", eic_sec_foreach_addr}, {
+	"func_foreach", eic_func_foreach}, {
+	"func_foreach_addr", eic_func_foreach_addr}, {
+	"func_foreach_code", eic_func_foreach_code}, {
+	"func_gen_args", eic_func_gen_args}, {
+	"func_gen_effect", eic_func_gen_effect}, {
+	"func_gen_locals", eic_func_gen_locals}, {
+	"func_gen_proto", eic_func_gen_proto}, {
+	/* VM routines */
 	"vm_add_regtbl_entry", eic_vm_add_regtbl_entry}, {
 	"vm_get_reg_name", eic_vm_get_reg_name}, {
 	"vm_get_reg_byname", eic_vm_get_reg_byname}, {
@@ -2362,15 +2477,6 @@ struct EIC_WRAPPER_TBL {
 	"vm_get_reg_type", eic_vm_get_reg_type}, {
 	"vm_get_num_regs", eic_vm_get_num_regs}, {
 	"vm_do_reg_op", eic_vm_do_reg_op}, {
-	"addr_foreach_xref", eic_addr_foreach_xref}, {
-	"sec_foreach", eic_sec_foreach}, {
-	"sec_foreach_addr", eic_sec_foreach_addr}, {
-	"func_foreach_addr", eic_func_foreach_addr}, {
-	"func_foreach_code", eic_func_foreach_code}, {
-	"func_gen_args", eic_func_gen_args}, {
-	"func_gen_effect", eic_func_gen_effect}, {
-	"func_gen_locals", eic_func_gen_locals}, {
-	"func_gen_proto", eic_func_gen_proto}, {
 	/* DB abstraction layer */
 	"db_schema_by_name", eic_db_schema_by_name}, {
 	"db_table_by_name", eic_db_table_by_name}, {
