@@ -10,16 +10,18 @@
 /* dammit ... had to add these for the IN_EIC error-trap flag */
 #include <bastard.h>
 extern struct DISASM_ENV disasm_env;
-struct EXT__ARCH *ext_arch;
-struct EXT__ASM *ext_asm;
-struct EXT__COMP *ext_comp;
-struct EXT__FORMAT  *ext_format;
-struct EXT__LANG	*ext_lang;
-struct EXT__OS *ext_os;
-struct EXT__PLUGIN *ext_plugin;
-struct EXT__ENGINE *ext_engine;
+struct EXT__ARCH *ext_arch = NULL;
+struct EXT__ASM *ext_asm = NULL;
+struct EXT__COMP *ext_comp = NULL;
+struct EXT__FORMAT  *ext_format = NULL;
+struct EXT__LANG	*ext_lang = NULL;
+struct EXT__OS *ext_os = NULL;
+struct EXT__PLUGIN *ext_plugin = NULL, *ext_plugin_list = NULL;
+struct EXT__ENGINE *ext_engine = NULL;
 /* </DAMMIT> */
 
+int ext_plugin_exists( char *filename ); /* forward decl */
+
 /* the way it works:
  * Extensions are loaded and unloaded by referring to their type constants.
  * Only routines which call the extensions will refer to the extension
@@ -28,6 +30,50 @@ struct EXT__ENGINE *ext_engine;
  * Note that the functions each extension provides will be callable via
  * the bastard API, so scripts will not call the extensions directly.
  */
+void ext_fprint_versions( FILE *f ) {
+	struct EXT__PLUGIN *plugin;
+
+	if ( ! ext_arch || ! ext_asm || ! ext_comp ||
+		! ext_format || ! ext_lang || !ext_os   )
+		return;
+
+	fprintf( f, "\text_arch %s: %2.2f\n", ext_arch->ext.filename, 
+			ext_arch->ext.version );
+	fprintf( f, "\text_asm %s: %2.2f\n", ext_asm->ext.filename, 
+			ext_asm->ext.version );
+	fprintf( f, "\text_comp %s: %2.2f\n", ext_comp->ext.filename, 
+			ext_comp->ext.version );
+	fprintf( f, "\text_format %s: %2.2f\n", ext_format->ext.filename, 
+			ext_format->ext.version );
+	fprintf( f, "\text_lang %s: %2.2f\n", ext_lang->ext.filename, 
+			ext_lang->ext.version );
+	fprintf( f, "\text_os %s: %2.2f\n", ext_os->ext.filename, 
+			ext_os->ext.version );
+
+	if ( ext_plugin_list ) {
+		plugin = ext_plugin_list->next;
+		while ( plugin ) {
+			fprintf(f, "\tplugin %s: %2.2f\n", plugin->ext.filename,
+				plugin->ext.version );
+			plugin = plugin->next;
+		}
+	}
+	return;
+}
+int ext_gen_filename( char *ext, char *name, char *buf, int buf_len ) {
+	struct stat statbuf;
+	snprintf( buf, buf_len, "%s/%s/%s.bc", env_get_share(), ext,
+		name );
+	if (stat(buf, &statbuf)) {
+		snprintf( buf, buf_len, "%s/%s/lib%s.so", env_get_share(),
+			ext, name );
+		if (stat(buf, &statbuf)) {
+			return(sys_set_lasterr(4594));
+		}
+	}
+	return( strlen(buf) );
+}
+
 void sys_init_extensions()
 {
 	ext_arch = calloc(sizeof (struct EXT__ARCH), 1);
@@ -37,13 +83,16 @@ void sys_init_extensions()
 	ext_comp = calloc(sizeof (struct EXT__COMP), 1);
 	ext_os = calloc(sizeof (struct EXT__OS), 1);
 	ext_engine = calloc(sizeof (struct EXT__ENGINE), 1);
-	ext_plugin = calloc(sizeof (struct EXT__PLUGIN), 1);
+	ext_plugin = ext_plugin_list = NULL;
 
 	return;
 }
 
 void cleanup_extensions()
 {
+	struct EXT__PLUGIN *ptr;
+	/* prevent attempts to free() twice */
+	if ( ! ext_arch ) 	return;
 	/* foreach extension, call cleanup and unload */
 	unload_extension(EXT_ARCH);
 	unload_extension(EXT_ASM);
@@ -52,7 +101,7 @@ void cleanup_extensions()
 	unload_extension(EXT_COMP);
 	unload_extension(EXT_OS);
 	unload_extension(EXT_ENGINE);
-	unload_extension(EXT_PLUGIN);
+
 	/* free the demn memory */
 	free(ext_arch);
 	free(ext_asm);
@@ -61,21 +110,46 @@ void cleanup_extensions()
 	free(ext_comp);
 	free(ext_os);
 	free(ext_engine);
-	free(ext_plugin);
+
+	/* cleanup plugins */
+/*	if ( ext_plugin_list ) {
+		ptr = ext_plugin_list->next;
+		while ( ptr ) {
+			ext_plugin_unload( ptr->name );
+			ptr = ext_plugin_list->next;
+		}
+		ext_plugin_unload( ext_plugin_list->name );
+	}
+*/	
+	ext_arch = NULL; ext_asm = NULL; ext_format = NULL;
+	ext_lang = NULL; ext_comp = NULL; ext_os = NULL;
+	ext_engine = NULL; ext_plugin = NULL;
+
 	return;
 }
 
-int loadext_internal(struct EXTENSION *ext, int type, char *filename,
+static int loadext_internal(struct EXTENSION *ext, int type, char *filename,
 		     int method)
 {
 
 	/* if previously loaded, just return */
-	//DEBUG_PrintMsg("checking if %s is already loaded\n", filename);
+	/* NO!! this prevents us from changing options!
+	DEBUG_PrintMsg("checking if %s is already loaded\n", filename);
 	if (ext->filename && !strcmp(filename, ext->filename))
 		return (1);
+	*/
+	if ( type == EXT_PLUGIN  && ext_plugin_exists( filename ) ){
+		return(3);
+	}
 	/* if another extension of the same type is currently loaded, unload it */
-	if (ext->filename || ext->lib)
+	/* NOTE: disabling this is a total hack to get libMAGIC working
+	 *       [since libMAGIC replaces itself with another extension]
+	 *       For now the hack is [wisely] limitged to EXT_FORMAT.
+	 *            if (ext->filename || ext->lib)
+	 */
+	if ((ext->filename || ext->lib) && type != EXT_FORMAT)
 		unload_extension(type);
+
 	/* if a library, load it */
 	if (method == EXT_SO)
 		if (!(ext->lib = dlopen(filename, RTLD_NOW))) {
@@ -92,7 +166,7 @@ int load_extension(int type, char *filename, void *param)
 
 	/* determine extension type based on file extension */
 	len = strlen(filename);
-	//DEBUG_PrintMsg("Starting to Load %s\n", filename);
+	DEBUG_PrintMsg("Starting to Load %s\n", filename);
 	if (!strncmp(".bc", &filename[len - 3], 3))
 		method = EXT_BC;	/* we have a script */
 	else {			/* binary extension is therefore arbitrary */
@@ -158,6 +232,7 @@ int load_extension(int type, char *filename, void *param)
 			ext->fn_init = &arch_init_stub;
 			ext->fn_cleanup = &arch_cleanup_stub;
 			ext_arch->fn_disasm_addr = &disasm_addr_stub;
+			ext_arch->fn_disasm_inv = &disasm_inv_stub;
 			ext_arch->fn_code_pat = &code_pat_stub;
 			ext_arch->fn_gen_effect = &gen_effect_stub;
 			ext_arch->fn_gen_int = &gen_int_stub;
@@ -166,6 +241,8 @@ int load_extension(int type, char *filename, void *param)
 			ext->fn_cleanup = dlsym(ext->lib, "ext_arch_cleanup");
 			ext_arch->fn_disasm_addr =
 			    dlsym(ext->lib, "disasm_addr");
+			ext_arch->fn_disasm_inv =
+			    dlsym(ext->lib, "disasm_invariant");
 			ext_arch->fn_code_pat =
 			    dlsym(ext->lib, "test_for_code_pattern");
 			ext_arch->fn_gen_effect =
@@ -280,20 +357,17 @@ int load_extension(int type, char *filename, void *param)
 		return (0);	/* this is an unknown type */
 	};
 
-	/*if (method == 0) { // reuse method for bailout 
-	   if (ext->lib) dlclose(ext->lib);
-	   return(0);
-	   } */
 	/* set filename, etc for extension */
 	ext->filename = calloc(len + 1, 1);
 	strncpy(ext->filename, filename, len);
-	ext->flags = method & type & EXT_OPEN;
+	ext->flags = method | type | EXT_OPEN;
+	ext->version = BASTARD_VERSION;	/* default version number */
 
 	/* if init call init */
 	DEBUG_PrintMsg("Calling init for %s\n", filename);
 	if (ext->fn_init)
 		ext->fn_init(param);
-	ext->flags &= EXT_INIT;
+	ext->flags = (ext->flags & ~EXT_STATUS_MASK) | EXT_INIT;
 	DEBUG_PrintMsg("Done Loading %s\n", filename);
 	return (1);
 }
@@ -338,17 +412,128 @@ int unload_extension(int type)
 		return (0);
 	/* if cleanup call cleanup */
 	DEBUG_PrintMsg("Unloading extension %s\n", ext->filename);
-	ext->flags &= EXT_CLEANUP;
+	ext->flags = (ext->flags & ~EXT_STATUS_MASK) | EXT_CLEANUP;
 	if (ext->fn_cleanup)
 		ext->fn_cleanup();
 	free(ext->filename);
 	if (ext->lib)
 		dlclose(ext->lib);
-	ext->flags &= EXT_CLOSED;
+	ext->flags = (ext->flags & ~EXT_STATUS_MASK) | EXT_CLOSED;
 	memset(ext, 0, sizeof (struct EXTENSION));
 	return (1);
 }
 
+/* plugin routines */
+static int ext_plugin_name( struct EXT__PLUGIN *plug, char *filename ){
+	char *ptr;
+	ptr = strrchr(filename, '/');
+	if ( ptr ) 
+		ptr++;
+	else
+		ptr = filename;
+	strncpy( plug->name, ptr, 63 );
+	return( strlen(plug->name) );
+}
+
+static struct EXT__PLUGIN *ext_new_plugin( void) {
+	struct EXT__PLUGIN *plug, *ptr;
+	plug = calloc(sizeof (struct EXT__PLUGIN), 1);
+	if ( plug ) {
+		/* insert into linked list */
+		plug->next = ext_plugin_list;
+		if ( ext_plugin_list ) 
+			ext_plugin_list->prev = plug;
+		/* set current plugin to new one */
+		ext_plugin = ext_plugin_list = plug;
+	}
+	return( plug );
+}
+
+static struct EXT__PLUGIN * ext_plugin_find_filename( char *filename ) {
+	struct EXTENSION *ext;
+	struct EXT__PLUGIN *ptr;
+	ptr = ext_plugin_list;
+	while ( ptr ) {
+		ext = &ptr->ext;
+		if ( ext->filename && ! strcmp( ext->filename, filename ) )
+			return( ptr );
+		ptr = ptr->next;
+	}
+	return(NULL);
+}
+
+int ext_plugin_exists( char *filename ) {
+	struct EXT__PLUGIN *ptr;
+	ptr = ext_plugin_find_filename( filename );
+	if ( ptr )
+		return(1);
+	return(0);
+}
+
+char * ext_plugin_load( char *filename, int options ) {
+	struct EXT__PLUGIN *plug;
+	plug = ext_plugin_find_filename( filename );
+	if (! plug ) {
+		plug = ext_new_plugin( );
+		ext_plugin_name( plug, filename );
+		plug->options = options;
+		/* ext_plugin now set to new plugin; pass 'plug' as param */
+		load_extension( EXT_PLUGIN, filename, plug );
+	} 
+		
+	return( plug->name );
+}
+
+static struct EXT__PLUGIN *ext_find_plugin( char *name ) {
+	struct EXT__PLUGIN *ptr;
+	ptr = ext_plugin_list;
+	while ( ptr ) {
+		if ( ! strncmp( ptr->name, name, 64 ) )
+			return( ptr );
+		ptr = ptr->next;
+	}
+	return(NULL);
+}
+
+static int ext_plugin_change( char *name ) {
+	/* change current plugin */
+	struct EXT__PLUGIN * plug;
+	plug = ext_find_plugin( name );
+	if ( plug ) {
+		ext_plugin = plug;
+		return(1);
+	}
+	return(0);
+}
+
+int ext_plugin_unload( char *name ) {
+	struct EXT__PLUGIN * plug, *ptr;
+	struct EXTENSION *ext;
+
+	plug = ext_find_plugin( name );
+	if ( ! plug )	return(0);
+	ext = &ext_plugin->ext;
+
+	/* change current plugin */
+	if ( ext_plugin == plug )
+		ext_plugin = plug->next;
+	/* remove from list */
+	if ( plug == ext_plugin_list ) {
+		ext_plugin_list = plug->next;
+	} else {
+		plug->prev->next = plug->next;
+		plug->next->prev = plug->prev;
+	}
+	/* perform cleanup */
+	if (ext->fn_cleanup)
+		ext->fn_cleanup();
+	free(ext->filename);
+	if (ext->lib)
+		dlclose(ext->lib);
+	free( plug );
+	return(1);
+}
+
 /* wrappers ------------- */
 /* these are used call .bc script routines instead of .so routines */
 
@@ -396,7 +581,7 @@ int read_header_stub(void)
 
 	EiC_parseString(":reset here");
 	sys_load_script(ext_format->ext.filename);
-	DEBUG_PrintMsg("executing ReadHeader for  %s\n",
+	DEBUG_PrintMsg("executing read_header for  %s\n",
 		       ext_format->ext.filename);
 
 	EiC_parseString("int rv @ %ld;", (long) &retval);
@@ -441,6 +626,21 @@ int disasm_addr_stub(unsigned char *c, int n, struct code *inst, long rva)
 	return (1);
 }
 
+int disasm_inv_stub(unsigned char *in, int in_len, struct ARCH_INVARIANT *inv)
+{
+	EiC_parseString(":reset here");
+	sys_load_script(ext_arch->ext.filename);
+	DEBUG_PrintMsg("executing %s\n", ext_arch->ext.filename);
+
+	EiC_parseString("unsigned char * unsafe c @ %ld;", (long) in);
+	EiC_parseString("int n @ %ld;", (long) &in_len);
+	EiC_parseString("struct ARCH_INVARIANT * unsafe inv @ %ld;", (long) inv);
+	EiC_parseString("disasm_invariant(in, in_len, inv);");
+
+	EiCp_ResetStart();
+	return (1);
+}
+
 int code_pat_stub(unsigned long rva, int pattern)
 {
 	int rv;
@@ -592,14 +792,14 @@ int sprint_section_end_stub(char *str, int len, char *name)
 	return (1);
 }
 
-int gen_asmfile_stub(char *path)
+int gen_asmfile_stub(FILE *f)
 {
 	EiC_parseString(":reset here");
 	sys_load_script(ext_asm->ext.filename);
 	DEBUG_PrintMsg("executing %s\n", ext_asm->ext.filename);
 
-	EiC_parseString("char * unsafe path @ %ld;", (long) path);
-	EiC_parseString("gen_file(path);");
+	EiC_parseString("FILE * unsafe f @ %ld;", (long) f);
+	EiC_parseString("gen_file(f);");
 
 	EiCp_ResetStart();
 	return (1);
@@ -685,7 +885,8 @@ void comp_cleanup_stub(void)
 	EiCp_ResetStart();
 	return;
 }
-int findstr_stub(char *buf, char *text, int text_len, int type)
+int findstr_stub(char *buf, int len, int *pos, char *text, int text_len, 
+			int type)
 {
 	EiC_parseString(":reset here");
 	sys_load_script(ext_comp->ext.filename);
@@ -693,9 +894,11 @@ int findstr_stub(char *buf, char *text, int text_len, int type)
 
 	EiC_parseString("char * unsafe buf @ %ld;", (long) buf);
 	EiC_parseString("char * unsafe text @ %ld;", (long) text);
+	EiC_parseString("int * unsafe pos @ %ld;", (long) pos);
 	EiC_parseString("int text_len @ %ld;", (long) &text_len);
+	EiC_parseString("int len @ %ld;", (long) &len);
 	EiC_parseString("int type @ %ld;", (long) &type);
-	EiC_parseString("findstr(buf, text, text_len, type);");
+	EiC_parseString("findstr(buf, len, pos, text, text_len, type);");
 	EiCp_ResetStart();
 	return (1);
 }
@@ -815,24 +1018,55 @@ int gen_final_stub(int func_id)
 	EiCp_ResetStart();
 	return (1);
 }
-int gen_hllfile_stub(char *path)
+int gen_hllfile_stub(FILE *f)
 {
 	EiC_parseString(":reset here");
 	sys_load_script(ext_lang->ext.filename);
 	DEBUG_PrintMsg("executing %s\n", ext_lang->ext.filename);
 
-	EiC_parseString("char * unsafe path @ %ld;", (long) path);
-	EiC_parseString("gen_file(path);");
+	EiC_parseString("FILE * unsafe f @ %ld;", (long) f);
+	EiC_parseString("gen_file(f);");
 
 	EiCp_ResetStart();
 	return (1);
 }
 
 /* safety wrappers -- called by routines outside of this file */
-int ext_plugin_main(void *param)
+int ext_plugin_exec( char *plugin, char *name, void *param ) {
+	char buf[64];
+	int (*plugin_fn)(void *);
+	struct EXTENSION *e;
+
+	if ( ! plugin )	return(0);
+	ext_plugin_change( plugin );
+	e = &ext_plugin->ext;
+
+	if ( (e->flags & EXT_METHOD_MASK) == EXT_SO ) {
+		plugin_fn = dlsym(e->lib, name);
+		if ( plugin_fn ){
+			return( (*plugin_fn)(param) );
+		}
+	} else {
+		EiC_parseString(":reset here");
+		sys_load_script(e->filename);
+		DEBUG_PrintMsg("executing %s\n", e->filename);
+
+		EiC_parseString("void * unsafe param @ %ld;", (long) param);
+		snprintf( buf, 64, "%s(param);", name );
+		EiC_parseString(buf);
+		EiCp_ResetStart();
+	}
+	return(1);
+}
+
+int ext_plugin_main(char *plugin, void *param)
 {
 	/* note: this one does all pre/post engines for plug-in */
 	int rv = 0;
+
+	if ( ! plugin )	return(0);
+	ext_plugin_change( plugin );
+
 	if (ext_plugin->fn_main) {
 		do_engine_stage(ext_plugin->name, "pre", (int) param);
 		rv = (*ext_plugin->fn_main) (param);
@@ -855,6 +1089,14 @@ int ext_disasm_addr(char *buf, char tbl, struct code *c, long rva)
 	return (0);
 }
 
+int ext_disasm_invariant(unsigned char *in, int in_len, 
+				 struct ARCH_INVARIANT *inv)
+{
+	if (ext_arch->fn_disasm_inv)
+		return ((*ext_arch->fn_disasm_inv) (in, in_len, inv));
+	return (0);
+}
+
 int ext_code_pat(int rva, int type)
 {
 	if (ext_arch->fn_code_pat)
@@ -946,10 +1188,10 @@ int ext_gen_final(int func_id)
 	return (0);
 }
 
-int ext_gen_hll_file(char *path)
+int ext_gen_hll_file(FILE *f)
 {
 	if (ext_lang->fn_gen_file)
-		return ((*ext_lang->fn_gen_file) (path));
+		return ((*ext_lang->fn_gen_file) (f));
 	return (0);
 }
 int ext_read_header(void)
@@ -1005,10 +1247,10 @@ int ext_sprint_sec_end(char *str, int len, char *name)
 	return (0);
 }
 
-int ext_gen_asm_file(char *path)
+int ext_gen_asm_file(FILE *f)
 {
 	if (ext_asm->fn_gen_file)
-		return ((*ext_asm->fn_gen_file) (path));
+		return ((*ext_asm->fn_gen_file)(f));
 	return (0);
 }
 
@@ -1017,11 +1259,12 @@ int do_engine(char *plugin, char *prefix, int num, int options)
 	char tmp[PATH_MAX];
 	struct stat statbuf;
 
-	sprintf(tmp, "%s/engines/%s/%s.%d.bc", disasm_env.home, plugin, prefix,
+	sprintf(tmp, "%s/engines/%s/%s.%d.bc", disasm_env.share, plugin, prefix,
 		num);
 	if (stat(tmp, &statbuf) || !load_extension(EXT_ENGINE, tmp, ext_engine))
 		return (sys_set_lasterr(4593));
-	return (ext_engine_main(&options));
+	/* engines have no 'init' and thus are never passed ext_engine */
+	return (ext_engine_main((void *)options));
 }
 
 int get_engine_num(char *str)
@@ -1029,6 +1272,7 @@ int get_engine_num(char *str)
 	int x;
 	char *end, tmp[16] = { 0 };
 
+	str = strrchr(str, '/');
 	end = strstr(str, ".bc") - 1;
 	x = end - str;
 	while (x && str[x] != '.')
@@ -1043,7 +1287,7 @@ int do_engine_stage(char *plugin, char *stage, int options)
 	glob_t globbuf;
 	int x, num;
 
-	sprintf(tmp, "%s/engines/%s/%s.*.bc", disasm_env.home, plugin, stage);
+	sprintf(tmp, "%s/engines/%s/%s.*.bc", disasm_env.share, plugin, stage);
 	glob(tmp, 0, NULL, &globbuf);
 	for (x = 0; x < globbuf.gl_pathc; x++) {
 		num = get_engine_num(globbuf.gl_pathv[x]);
