@@ -104,9 +104,11 @@ typedef int (*new_imp_fn) (long, char *, char *, int);
 typedef int (*new_lib_fn) (char *, int, int);
 typedef int (*new_fun_fn) (long, char *, int, int);
 typedef int (*new_sec_fn) (char *, long, int, long, int);
+typedef int (*struct_fn) (long, unsigned long);
 
 struct ELF_INFO {
 	int fd;			/* file descriptor of target */
+	long base;		/* base of ELF file in fd */
 	/* these are fn pointers instead of calls to bastard API */
 	new_addr_fn addr_new;
 	new_name_fn name_new;
@@ -114,6 +116,7 @@ struct ELF_INFO {
 	new_lib_fn lib_new;
 	new_fun_fn func_new;
 	new_sec_fn sec_new;
+	struct_fn  struct_apply;
 	/* this is a collection of string buffers */
 	struct ELF_STRING_BUF str;
 	/* these are return values from parse_elf() */
@@ -138,6 +141,7 @@ struct ELF_INFO {
 /* --------------------------------------------------------------- globals */
 char strMagic[4] = { 0x7F, 'E', 'L', 'F' };
 char *e_strtable = NULL;
+int e_strtab_size;
 char buf[512], fbuf[8];		/* temporary storage */
 
 /* ----------------------------------------- Constant-to-String Conversions */
@@ -381,18 +385,30 @@ const char *get_elf_data(int ei_data)
 }
 
 /* ---------------------------------------------------- UHHHHHH */
-struct ELF_PROG **g_pht;	/* internal 'global' copy of pht */
+struct ELF_SECT **g_sht = NULL;	/* internal 'global' copy of sht */
+struct ELF_PROG **g_pht = NULL;	/* internal 'global' copy of pht */
+struct ELF_SECT **create_sht(struct ELF_HDR *hdr); /* fwd decl */
 struct ELF_PROG **create_pht(struct ELF_HDR *hdr); /* fwd decl */
 
 int get_file_offset(int addr, struct ELF_HDR *hdr)
 {
+	struct ELF_PROG *p;
+	struct ELF_SECT *s;
 	int x;
 	if (! g_pht) g_pht = create_pht(hdr);
 	for (x = 0; x < hdr->e_phnum; x++) {
-		if (addr >= g_pht[x]->p_vaddr &&
-		    addr < (g_pht[x]->p_vaddr + g_pht[x]->p_filesz)) {
-			return ((addr - g_pht[x]->p_vaddr) +
-				g_pht[x]->p_offset);
+		p = g_pht[x];
+		if (addr >= p->p_vaddr && addr < (p->p_vaddr + p->p_filesz)) {
+			return ((addr - p->p_vaddr) + p->p_offset);
+		}
+	}
+
+	/* .a files have no pht; we do those here */
+	if (! g_sht) g_sht = create_sht(hdr);
+	for (x = 0; g_sht[x] && x < hdr->e_shnum; x++) {
+		s = g_sht[x];
+		if (addr >= s->sh_addr && addr < (s->sh_addr + s->sh_size)) {
+			return ((addr - s->sh_addr) + s->sh_offset);
 		}
 	}
 	return (0);
@@ -408,7 +424,7 @@ struct ELF_SYM **create_symt(int offset, int count)
 
 	for (x = 0; x < count; x++) {
 		symt[x] = malloc(sizeof (struct ELF_SYM));
-		lseek(opt->fd, offset + (sizeof (struct ELF_SYM) * x), 0);
+		lseek(opt->fd, opt->base + offset + (sizeof (struct ELF_SYM) * x), SEEK_SET);
 		read(opt->fd, symt[x], sizeof (struct ELF_SYM));
 	}
 	return (symt);
@@ -441,8 +457,7 @@ int do_symtab(int sym_addr, int sym_size, char *strtable, struct DYNSTR *libs,
 	if (!strtable || !sym_addr)
 		return (0);
 
-	sym = malloc(sizeof (struct ELF_SYM));
-	lseek(opt->fd, sym_addr, 0);
+	lseek(opt->fd, opt->base + sym_addr, SEEK_SET);
 	count = sym_size / sizeof (struct ELF_SYM);
 	symt = create_symt(sym_addr, count);
 
@@ -454,48 +469,45 @@ int do_symtab(int sym_addr, int sym_size, char *strtable, struct DYNSTR *libs,
 		if (sym->st_name > 0 && c[0] > 31 && c[0] < 127) {
 			switch ((sym->st_info & 0xf)) {
 			case STT_OBJECT:	/* symbol is a DATA OBJECT */
+
 				(*opt->addr_new) (sym->st_value, sym->st_size,
 						  get_file_offset(sym->st_value,
 								  hdr),
-						  ADDR_SYMBOL & ADDR_DATA);
+						  ADDR_SYMBOL | ADDR_DATA);
 				(*opt->name_new) (sym->st_value,
 						  strtable + sym->st_name,
-						  0x21);
+						  NAME_AUTO | NAME_SYMBOL);
 				break;
 			case STT_FUNC:	/* Symbol is a FUNCTION */
-				(*opt->addr_new) (sym->st_value, sym->st_size, sym->st_value,
-						  ADDR_SYMBOL & ADDR_CODE);
-				
-				    (*opt->name_new) (sym->st_value,
-						      strtable + sym->st_name,
-						      0x23);
-				if (sym->st_shndx == 0 && libs) {
+				/* create an address so we have real PA */
+				(*opt->addr_new) (sym->st_value, sym->st_size,
+						  get_file_offset(sym->st_value, hdr),
+						  ADDR_SYMBOL | ADDR_CODE);
+				if (! sym->st_shndx ) {
 					/* this is an imported function */
-					/* check to see which DT_NEEDED this falls under */
+					/* check which DT_NEEDED this falls under */
 					tmp = NULL;
-					pLIB = libs->next;
-					while (pLIB) {
-						if (sym->st_name > pLIB->offset) {
-							if (!pLIB->next ||
-							    pLIB->next->offset >
-							    sym->st_name) {
-								tmp =
-								    strtable +
-								    pLIB->
-								    offset;
+					if ( libs ) {
+						pLIB = libs->next;
+						while (pLIB) {
+							if (sym->st_name > pLIB->offset) {
+								if (!pLIB->next ||
+							    	     pLIB->next->offset >
+								     sym->st_name) {
+									tmp =
+								    	strtable + pLIB->offset;
+								}
 							}
+							pLIB = pLIB->next;
 						}
-						pLIB = pLIB->next;
 					}
 					/* tmp == DT_NEEDED == library owning this func */
-					
-					    (*opt->imp_new) (sym->st_value, tmp,
+					(*opt->imp_new) (sym->st_value, tmp,
 							     strtable +
 							     sym->st_name, 0);
 				} else {
 					/* Not an imported function */
-					
-					    (*opt->func_new) (sym->st_value,
+					(*opt->func_new) (sym->st_value,
 							      strtable +
 							      sym->st_name,
 							      sym->st_size,
@@ -509,10 +521,19 @@ int do_symtab(int sym_addr, int sym_size, char *strtable, struct DYNSTR *libs,
 			case STT_SECTION:
 			case STT_NOTYPE:
 				/* add extra symbols [those w/o rva's are discarded] */
-				
-				    (*opt->name_new) (sym->st_value,
+				if ( sym->st_shndx ) {
+					(*opt->addr_new) (sym->st_value, sym->st_size,
+						  get_file_offset(sym->st_value, hdr),
+						  ADDR_SYMBOL | ADDR_DATA);
+					(*opt->name_new) (sym->st_value,
 						      strtable + sym->st_name,
-						      0x20);
+						      NAME_AUTO | NAME_SYMBOL);
+				} else {
+				/* note: those with no shndx are imports */
+					(*opt->imp_new) (sym->st_value, NULL,
+							     strtable +
+							     sym->st_name, 0);
+				}
 			case STT_LOPROC:
 			case STT_HIPROC:
 			default:
@@ -535,8 +556,7 @@ struct ELF_DYN **create_dynt(struct ELF_PROG *phdr, int count)
 
 	for (x = 0; x < count; x++) {
 		dynt[x] = malloc(sizeof (struct ELF_DYN));
-		lseek(opt->fd, phdr->p_offset + (sizeof (struct ELF_DYN) * x),
-		      0);
+		lseek(opt->fd, opt->base + phdr->p_offset + (sizeof (struct ELF_DYN) * x), SEEK_SET);
 		read(opt->fd, dynt[x], sizeof (struct ELF_DYN));
 	}
 	return (dynt);
@@ -606,7 +626,7 @@ int add_dyn_info(struct ELF_DYNAMIC_INFO *info, struct ELF_DYN *dyn)
 		sprintf(buf, ".init routine at %08X\n", dyn->val);
 		strncat(opt->str.note, buf, opt->str.n_len);
 		(*opt->addr_new) (dyn->val, 1, dyn->val,
-				  ADDR_SYMBOL & ADDR_CODE);
+				  ADDR_SYMBOL | ADDR_CODE);
 		//(*opt->name_new)( dyn->val, "elf_init", 0x23);      
 		(*opt->func_new) (dyn->val, "elf_init",
 				  0, NAME_AUTO | NAME_SYMBOL);
@@ -616,7 +636,7 @@ int add_dyn_info(struct ELF_DYNAMIC_INFO *info, struct ELF_DYN *dyn)
 		sprintf(buf, ".fini routine at %08X\n", dyn->val);
 		strncat(opt->str.note, buf, opt->str.n_len);
 		(*opt->addr_new) (dyn->val, 1, dyn->val,
-				  ADDR_SYMBOL & ADDR_CODE);
+				  ADDR_SYMBOL | ADDR_CODE);
 		//(*opt->name_new)( dyn->val, "elf_fini", 0x23);      
 		(*opt->func_new) (dyn->val, "elf_fini",
 				  0, NAME_AUTO | NAME_SYMBOL);
@@ -696,7 +716,7 @@ int do_dynamic(struct ELF_PROG *phdr, struct ELF_HDR *hdr)
 
 		/* load string table */
 		strtab = malloc(opt->dynamic.str_size);
-		lseek(opt->fd, symtab, 0);
+		lseek(opt->fd, opt->base + symtab, SEEK_SET);
 		read(opt->fd, strtab, opt->dynamic.str_size);
 	}
 
@@ -797,7 +817,9 @@ struct ELF_SECT **create_sht(struct ELF_HDR *hdr)
 	sht = calloc(sizeof (void *), hdr->e_shnum);
 	for (x = 0; x < hdr->e_shnum; x++) {
 		sht[x] = malloc(sizeof (struct ELF_SECT));
-		lseek(opt->fd, hdr->e_shoff + (hdr->e_shentsize * x), 0);
+		if ( ! sht[x] ) return(sht);
+		lseek(opt->fd, opt->base + hdr->e_shoff + 
+			(hdr->e_shentsize * x), SEEK_SET);
 		read(opt->fd, sht[x], sizeof (struct ELF_SECT));
 	}
 	return (sht);
@@ -827,15 +849,17 @@ int do_sht(struct ELF_HDR *hdr)
 	struct ELF_SECT *shdr = NULL, *strtab = NULL, *symtab = NULL;
 	struct ELF_SECT **sht = NULL;
 
-	if (!hdr->e_shnum)
+	if (! hdr || !hdr->e_shnum)
 		return (0);
-	sht = create_sht(hdr);
+	if (! g_sht ) g_sht = create_sht(hdr);
+	sht = g_sht;
 
 	/* Load String Table */
 	if (hdr->e_shstrndx) {
 		shdr = sht[hdr->e_shstrndx];
 		e_strtable = malloc(shdr->sh_size);
-		lseek(opt->fd, shdr->sh_offset, 0);
+		e_strtab_size = shdr->sh_size;
+		lseek(opt->fd, opt->base + shdr->sh_offset, SEEK_SET);
 		read(opt->fd, e_strtable, shdr->sh_size);
 	}
 
@@ -843,8 +867,9 @@ int do_sht(struct ELF_HDR *hdr)
 	for (x = 0; x < hdr->e_shnum; x++) {
 		shdr = sht[x];
 
+		if (! shdr )	continue;
 		/* get section name from string table */
-		if (hdr->e_shstrndx)
+		if (hdr->e_shstrndx && shdr->sh_name < e_strtab_size)
 			sh_name = e_strtable + shdr->sh_name;
 		else
 			sh_name = "(null)";
@@ -924,14 +949,13 @@ int do_sht(struct ELF_HDR *hdr)
 	/* if a symbol and a string table were found, load symbols */
 	if (strtab && symtab) {
 		strtab_buf = malloc(strtab->sh_size);
-		lseek(opt->fd, strtab->sh_offset, 0);
+		lseek(opt->fd, opt->base + strtab->sh_offset, SEEK_SET);
 		read(opt->fd, strtab_buf, strtab->sh_size);
 		do_symtab(symtab->sh_offset, symtab->sh_size, strtab_buf, NULL,
 			  hdr);
 		free(strtab_buf);
 	}
 
-	free_sht(sht, hdr);
 
 	return (rv);
 }
@@ -947,7 +971,8 @@ struct ELF_PROG **create_pht(struct ELF_HDR *hdr)
 	pht = calloc(sizeof (void *), hdr->e_phnum);
 	for (x = 0; x < hdr->e_phnum; x++) {
 		pht[x] = malloc(sizeof (struct ELF_PROG));
-		lseek(opt->fd, hdr->e_phoff + (hdr->e_phentsize * x), 0);
+		if ( ! pht[x] ) return(pht);
+		lseek(opt->fd, opt->base + hdr->e_phoff + (hdr->e_phentsize * x), SEEK_SET);
 		read(opt->fd, pht[x], sizeof (struct ELF_PROG));
 	}
 	return (pht);
@@ -1032,7 +1057,6 @@ int do_pht(struct ELF_HDR *hdr, int create_sections)
 		do_dynamic(dyn_hdr, hdr);
 	}
 
-	free_pht(pht, hdr);
 	return (0);
 }
 
@@ -1048,6 +1072,9 @@ int do_elf_datatypes()
 	d4 = dtype_new("Elf32_Sword", 4, DT_SIGNED);
 	d5 = dtype_new("Elf32_Word", 4, DT_UNSIGNED);
 	d6 = dtype_get("unsigned char");
+	if (! d6 ) {
+		d6 = dtype_new("unsigned char", 1, DT_UNSIGNED);
+	}
 
 	/* create ELF Header struct */
 	e_struct = struct_new("Elf32_Ehdr", 52);
@@ -1068,8 +1095,9 @@ int do_elf_datatypes()
 	struct_add_member(e_struct, d2, 1, 14, "e_shstrndx");
 
 	/* apply this struct to the file itself */
-	//struct_apply( 0, e_struct );
-	return (0);
+	(*opt->addr_new) (0, 52, 0, ADDR_STRUCT | ADDR_DATA);
+	(*opt->struct_apply)( 0, e_struct );
+	return (1);
 }
 
 
@@ -1086,9 +1114,17 @@ int parse_elf(struct ELF_INFO *info)
 		return (EBADPARAM);
 	opt = info;
 	h = &opt->header;
+	/* clear globals in case we are called more than once :) */
+	g_sht = NULL;
+	g_pht = NULL;
+	e_strtable = NULL;
 
 	/*  ______Load the ELF File Header_______ */
-	read(opt->fd, h, sizeof (struct ELF_HDR));
+	lseek( opt->fd, opt->base, SEEK_SET);
+	if( read(opt->fd, h, sizeof (struct ELF_HDR)) 
+			< sizeof (struct ELF_HDR) ) {
+		return(0);
+	}
 
 	/* Check Magic Number */
 	if (strncmp(h->e_ident, strMagic, 4)) {
@@ -1114,5 +1150,7 @@ int parse_elf(struct ELF_INFO *info)
 		do_pht(h, 1);
 	}
 
+	if (g_sht) free_sht(g_sht, h);
+	if (g_pht) free_pht(g_pht, h);
 	return (1);
 }
