@@ -47,7 +47,7 @@ int gen_final(int func_id)
 	return (1);
 }
 /* generate a high-level language file for target */
-int gen_file(char *path)
+int gen_file(FILE *f)
 {
 	/* do data types */
 	/* for i in functions sprint_func */
