@@ -11,33 +11,37 @@ struct BSQL_SETTINGS *bsql_settings = &bsql_settings_global;
 /* ========================================================= JOINS */
 /* JOIN t1 and t2 based on table expression 'expr' */
 struct BSQL_TMPTBL * bsql_join_on( struct BSQL_TMPTBL *t1,
-					struct BSQL_TMPTBL *t2, void *expr  );
+					struct BSQL_TMPTBL *t2, void *expr  ){
+	return(NULL);
+}
 /* JOIN t1 and t2 using columns 'col' */
 struct BSQL_TMPTBL * bsql_join_using( struct BSQL_TMPTBL *t1,
-					struct BSQL_TMPTBL *t2, struct BSQL_COL *col  );
+					struct BSQL_TMPTBL *t2, struct BSQL_COL *col  ){
+	return(NULL);
+}
 /* JOIN t1 and t2 based on common columns */
 struct BSQL_TMPTBL * bsql_join_natural( struct BSQL_TMPTBL *t1, 
-						    struct BSQL_TMPTBL *t2  );
+						    struct BSQL_TMPTBL *t2  ){
+	return(NULL);
+}
 
 
-struct BSQL_SELECTION {
-};
-struct BSQL_COLUMNLIST {
-};
-struct BSQL_SEARCHCOND{
-};
-struct BSQL_TABLEEXP {
-};
 //SELECT table.field FROM table WHERE table.field = value
 int bsql_select_op() {
-
+	return(0);
 }
 //INSERT	INTO table ( field...) VALUES (val...)
-bsql_insert_op();
+int bsql_insert_op() {
+	return(0);
+}
 //UPDATE	table SET field = value WHERE field.table = value
-bsql_update_op();
+int bsql_update_op(){
+	return(0);
+}
 //DELETE	FROM table WHERE field.table = value
-bsql_delete_op();
+int bsql_delete_op(){
+	return(0);
+}
 
 /*
 	delete from table/index : if index do find, else iterate and 
@@ -47,8 +51,18 @@ bsql_delete_op();
 */
 
 //CURSOR   OPEN cursor; FETCH cursor INTO :x, :y; CLOSE cursor
-bsql_cursor_open();
-bsql_cursor_add();
-bsql_cursor_current();
-bsql_cursor_fetch();
-bsql_cursor_close();
+int bsql_cursor_open(){
+	return(0);
+}
+int bsql_cursor_add(){
+	return(0);
+}
+int bsql_cursor_current(){
+	return(0);
+}
+int bsql_cursor_fetch(){
+	return(0);
+}
+int bsql_cursor_close(){
+	return(0);
+}
