@@ -13,12 +13,13 @@ int plugin_main(void *param)
 
 	target = env_get_target();
 
+	ext_add_data_types( disasm_word_size() );
 	/* PREDISASM phase: parse the file header */
-	if (target->status < DISASM_TGT_PREDISASM)
+	if (! (target->status & DISASM_TGT_PREDISASM))
 		if (!target_apply_format())
 			return (sys_set_lasterr(4570));
 	/* Calling EiC from withing EiC really fucks the stack over */
-	target->status = DISASM_TGT_PREDISASM;
+	target->status |= DISASM_TGT_PREDISASM;
 
 
 	/* DISASM phase */
@@ -34,12 +35,12 @@ int plugin_main(void *param)
 		}
 		cont = bdb_index_next(SECTION_RVA, &s);
 	}
-	target->status = DISASM_TGT_DISASM;
+	target->status |= DISASM_TGT_DISASM;
 
 	/* POSTDISASM Phase */
 	/* create an export for the entry point */
 	exp_new(target->info.entry, "_start");
-	target->status = DISASM_TGT_POSTDISASM;
+	target->status |= DISASM_TGT_POSTDISASM;
 
 	sys_msg("Done.\n");
 	return (1);
