@@ -4,9 +4,9 @@
 
 
 /* OUPUT ASM: Use ASM extension to create .asm file */
-int plugin_main(char *filename)
+int plugin_main( FILE *f )
 {
-	if (!filename)
+	if (!f)
 		return (sys_set_lasterr(9010));
-	return (ext_gen_asm_file(filename));
+	return (ext_gen_asm_file(f));
 }
