@@ -7,18 +7,19 @@
 
 
 /* OUTOUT BINARY: create binary image of (patched) target */
-int plugin_main(char *filename)
+int plugin_main( FILE *f )
 {
 	int fd, count;
 	unsigned char buf[512];
 	struct DISASM_TGT *target = env_get_target();
 
-	if ( (fd = open(filename, O_WRONLY | O_CREAT | O_EXCL)) > 0 ) {
+	fd = fileno( f );
+	if ( fd > 2 ) {
+		/* do not write to STDOUT/ERR :) */
 		lseek( target->fd, 0, SEEK_SET );
 		while ( (count = read(target->fd, buf, 512)) > 0 ) {
 			write( fd, buf, count );
 		}
-		close( fd );
 	}
-	return (0);
+	return (1);
 }
