@@ -4,7 +4,7 @@
 
 
 /* OUTPUT DIFF : Create binary diff of (patched) target */
-int plugin_main(char *filename)
+int plugin_main(FILE *f)
 {
 	sys_msg("DIFF output not supported\n");
 	return (0);
