@@ -4,18 +4,14 @@
 
 
 /* OUTPUT HEX: Create hex dump of target */
-int plugin_main(char *filename)
+int plugin_main(FILE *dump_file)
 {
-	FILE *dumpFile;
 	unsigned int x, y;
 	unsigned char hex[48 + 4], ascii[16 + 4], c;
 	struct DISASM_TGT *target = env_get_target();
 
-	if (!filename)
+	if (!dump_file)
 		return (sys_set_lasterr(9010));
-	dumpFile = fopen(filename, "w");
-	if (!dumpFile)
-		return (sys_set_lasterr(9030));
 
 	for (x = 0; x < target->info.size; x += y) {
 		memset(hex, 0, 52);
@@ -33,8 +29,7 @@ int plugin_main(char *filename)
 			for (; y < 16; y++)
 				sprintf(hex, "%s   ", hex);
 
-		fprintf(dumpFile, "%08X : %s\t%s\n", x, hex, ascii);
+		fprintf(dump_file, "%08X : %s\t%s\n", x, hex, ascii);
 	}
-	fclose(dumpFile);
 	return (1);
 }
