@@ -4,7 +4,7 @@
 
 
 /* OUTPUT HTML : create HTML-formatted .lst file */
-int plugin_main(char *filename) {
+int plugin_main(FILE *f) {
 	sys_msg( "HTML output not supported\n");
 	return (0);
 }
