@@ -260,7 +260,7 @@ loadfile_dlg_format_activate             (GtkEditable     *editable,
    entry = GTK_ENTRY(lookup_widget(load_file_dlg.self, "lfd_format_entry")); 
 
    /* create options based on .dat file */
-   sprintf(filename, "%s/formats/%s.dat", env_get_home(), 
+   sprintf(filename, "%s/formats/%s.dat", env_get_share(), 
          gtk_entry_get_text(entry));
 
    create_dlg_from_datfile( filename,  &load_file_dlg.format);
@@ -287,7 +287,7 @@ loadfile_dlg_cpu_activate                (GtkEditable     *editable,
    entry = GTK_ENTRY(lookup_widget( load_file_dlg.self, "lfd_cpu_entry")); 
 
    /* create options based on .dat file */
-   sprintf(filename, "%s/arch/%s.dat", env_get_home(), 
+   sprintf(filename, "%s/arch/%s.dat", env_get_share(), 
          gtk_entry_get_text(entry));
 
    create_dlg_from_datfile( filename, &load_file_dlg.cpu);
@@ -423,7 +423,7 @@ loadfile_dlg_asm_activate                (GtkEditable     *editable,
 	
 
    /* create options based on .dat file */
-   sprintf(filename, "%s/asm/%s.dat", env_get_home(), 
+   sprintf(filename, "%s/asm/%s.dat", env_get_share(), 
          gtk_entry_get_text(entry));
 
    create_dlg_from_datfile( filename, &load_file_dlg.asmblr);
@@ -440,7 +440,7 @@ loadfile_dlg_hll_activate                (GtkEditable     *editable,
       GTK_ENTRY(lookup_widget(load_file_dlg.self, "lfd_hll_entry")); 
 
    /* create options based on .dat file */
-   sprintf(filename, "%s/lang/%s.dat", env_get_home(), 
+   sprintf(filename, "%s/lang/%s.dat", env_get_share(), 
          gtk_entry_get_text(entry));
 
    create_dlg_from_datfile( filename, &load_file_dlg.hll);
@@ -454,7 +454,7 @@ void loadfile_dlg_comp_activate(GtkEditable     *editable, gpointer         user
       GTK_ENTRY(lookup_widget(load_file_dlg.self, "lfd_comp_entry")); 
 
    /* create options based on .dat file */
-   sprintf(filename, "%s/comp/%s.dat", env_get_home(), 
+   sprintf(filename, "%s/comp/%s.dat", env_get_share(), 
          gtk_entry_get_text(entry));
 
    create_dlg_from_datfile( filename, &load_file_dlg.comp);
@@ -467,7 +467,7 @@ void loadfile_dlg_os_activate(GtkEditable     *editable, gpointer         user_d
       GTK_ENTRY(lookup_widget(load_file_dlg.self, "lfd_os_entry")); 
 
    /* create options based on .dat file */
-   sprintf(filename, "%s/os/%s.dat", env_get_home(), 
+   sprintf(filename, "%s/os/%s.dat", env_get_share(), 
          gtk_entry_get_text(entry));
 
    create_dlg_from_datfile( filename, &load_file_dlg.os);
