@@ -107,7 +107,7 @@ int init_load_file_dlg(){
 	gtk_signal_emit_by_name( (GtkObject *)combo->entry, "activate" );
 	
 	/* create disassembly options */	
-	sprintf(default_target, "%s/plugins", env_get_home());
+	sprintf(default_target, "%s/plugins", env_get_share());
 	create_disasm_dlg(default_target);
 	load_file_dlg.init = 1;
 	
